// Copyright (c) 2020 RISC Software GmbH
//
// This file was generated by CPACSGen from CPACS XML Schema (c) German Aerospace Center (DLR/SC).
// Do not edit, all changes are lost when files are re-generated.
//
// Licensed under the Apache License, Version 2.0 (the "License")
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#pragma once

#include <boost/optional.hpp>
#include <boost/utility/in_place_factory.hpp>
#include <CCPACSGenericSystems.h>
#include <string>
#include <tixi.h>
#include <typeinfo>
#include "CreateIfNotExists.h"
#include "CTiglError.h"
#include "tigl_internal.h"

namespace tigl
{
class CTiglUIDManager;
class CTiglUIDObject;
class CCPACSAircraftModel;
class CCPACSRotorcraftModel;

namespace generated
{
    // This class is used in:
    // CPACSAircraftModel
    // CPACSRotorcraftModel

    /// @brief systemsType
    /// 
    /// Systems type, containing the aircraft's control system
    /// data
    /// Please see the attached picture for further
    /// documentation
    /// @see controls
    /// 
    class CPACSSystems
    {
    public:
        TIGL_EXPORT CPACSSystems(CCPACSAircraftModel* parent, CTiglUIDManager* uidMgr);
        TIGL_EXPORT CPACSSystems(CCPACSRotorcraftModel* parent, CTiglUIDManager* uidMgr);

        TIGL_EXPORT virtual ~CPACSSystems();

        template<typename P>
        bool IsParent() const
        {
            return m_parentType != NULL && *m_parentType == typeid(P);
        }

        template<typename P>
        P* GetParent()
        {
#ifdef HAVE_STDIS_SAME
            static_assert(std::is_same<P, CCPACSAircraftModel>::value || std::is_same<P, CCPACSRotorcraftModel>::value, "template argument for P is not a parent class of CPACSSystems");
#endif
            if (!IsParent<P>()) {
                throw CTiglError("bad parent");
            }
            return static_cast<P*>(m_parent);
        }

        template<typename P>
        const P* GetParent() const
        {
#ifdef HAVE_STDIS_SAME
            static_assert(std::is_same<P, CCPACSAircraftModel>::value || std::is_same<P, CCPACSRotorcraftModel>::value, "template argument for P is not a parent class of CPACSSystems");
#endif
            if (!IsParent<P>()) {
                throw CTiglError("bad parent");
            }
            return static_cast<P*>(m_parent);
        }

        TIGL_EXPORT virtual CTiglUIDObject* GetNextUIDParent();
        TIGL_EXPORT virtual const CTiglUIDObject* GetNextUIDParent() const;

        TIGL_EXPORT CTiglUIDManager& GetUIDManager();
        TIGL_EXPORT const CTiglUIDManager& GetUIDManager() const;

        TIGL_EXPORT virtual void ReadCPACS(const TixiDocumentHandle& tixiHandle, const std::string& xpath);
        TIGL_EXPORT virtual void WriteCPACS(const TixiDocumentHandle& tixiHandle, const std::string& xpath) const;

        TIGL_EXPORT virtual const boost::optional<CCPACSGenericSystems>& GetGenericSystems() const;
        TIGL_EXPORT virtual boost::optional<CCPACSGenericSystems>& GetGenericSystems();

        TIGL_EXPORT virtual CCPACSGenericSystems& GetGenericSystems(CreateIfNotExistsTag);
        TIGL_EXPORT virtual void RemoveGenericSystems();

    protected:
        void* m_parent;
        const std::type_info* m_parentType;

        CTiglUIDManager* m_uidMgr;

        /// Node for geometrical layout of system components
        /// based on simple geometric shapes
        boost::optional<CCPACSGenericSystems> m_genericSystems;

    private:
        CPACSSystems(const CPACSSystems&) = delete;
        CPACSSystems& operator=(const CPACSSystems&) = delete;

        CPACSSystems(CPACSSystems&&) = delete;
        CPACSSystems& operator=(CPACSSystems&&) = delete;
    };
} // namespace generated

// CPACSSystems is customized, use type CCPACSACSystems directly
} // namespace tigl
