/* This file was automatically generated from tigl.h on 2022-03-14 */

/* 
* Copyright (C) 2007-2012 German Aerospace Center (DLR/SC)
*
* Created: 2012-08-13 Martin Siggel <martin.siggel@dlr.de>
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

/* Visual Studio C macros */
#ifdef _MSC_VER
  #ifndef _CRT_SECURE_NO_WARNINGS
    #define _CRT_SECURE_NO_WARNINGS
  #endif
  #pragma warning(disable: 4090)
  #if _MSC_VER < 1900
    #define snprintf _snprintf
  #endif
#endif

#include <math.h>
#include <matrix.h>
#include <mex.h>

#include <string.h>
#include <stdio.h>

#include "common.h"

#if defined(WIN32)
  #define MEX_EXPORT __declspec(dllexport)
#endif

/*
 * helping function to convert matlab array into c-string 
 */
int mxToString(const mxArray * mxStr, char ** cstr)
{
  size_t strLen = 0;

       /* Input must be a string. */
  if (mxIsChar(mxStr) != 1) {
    mexErrMsgTxt("Input must be a string.");
    return 1;
  }

  /* Input must be a row vector. */
  if (mxGetM(mxStr) > 1) {
    mexErrMsgTxt("Input must be a row vector.");
    return 1;
  }
    
  /* Get the length of the input string. */
  strLen = mxGetNumberOfElements(mxStr) + 1;

  /* Allocate memory for input and output strings. */
  *cstr = (char*) mxCalloc(strLen, sizeof(char));

  /* Copy the string data from prhs[0] into a C string 
   * input_buf. */
  return mxGetString(mxStr, *cstr, strLen);
}

mwSize mxGetArraySize(const mxArray * mxArr)
{
    mwSize ndims =  mxGetNumberOfDimensions(mxArr);
    mwSize idim;

    mwSize totalSize = 1;
    for (idim = 0; idim < ndims; ++idim) {
        totalSize *= mxGetDimensions(mxArr)[idim];
    }
    
    return totalSize;
}

int mxToInt(const mxArray * mxArr)
{
    double dv = *mxGetPr(mxArr);
    if(!isinteger(dv))
        mexWarnMsgTxt("Argument is not an integer\n");

    return (int)dv;
}

size_t mxToSize_t(const mxArray * mxArr)
{
    double dv = *mxGetPr(mxArr);
    if(!isinteger(dv))
        mexWarnMsgTxt("Argument is not an integer\n");

    return (size_t)dv;
}

void mxToIntArray(const mxArray * mxArr, int ** pIntArray)
{
    mwSize i, totalSize;
    totalSize = mxGetArraySize(mxArr);
    
    *pIntArray = (int*) mxMalloc(sizeof(int)*totalSize);
    for (i = 0; i< totalSize; ++i) {
        (*pIntArray)[i] = (int)mxGetPr(mxArr)[i];
    }
}

mxArray * iArrayToMx(const int * array, int size)
{
    mxArray * out = mxCreateDoubleMatrix(1,size, mxREAL);
    double * rawArray = mxGetPr(out);
    int i;
    for (i = 0; i < size; ++i) {
        rawArray[i] = (double) array[i];
    }
    
    return out;
}

mxArray * dArrayToMx(const double * array, int size)
{
    mxArray * out = mxCreateDoubleMatrix(1,size, mxREAL);
    double *  rawArray = mxGetPr(out);
    int i;
    for (i = 0; i < size; ++i) {
        rawArray[i] = array[i];
    }
    
    return out;
}


#include <tigl.h>

void handleError(const char * funcname, TiglReturnCode code)
{
    if (code != TIGL_SUCCESS) {
        char errormsg[1024];
        sprintf(errormsg, "Error: \"%s\" returned code %d", funcname, code);
        mexErrMsgTxt(errormsg);
    }
}

void mex_tiglOpenCPACSConfiguration(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TixiDocumentHandle tixiHandle;
    char* configurationUID = NULL;

    /* output variables */
    TiglCPACSConfigurationHandle cpacsHandlePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglOpenCPACSConfiguration(tixiHandle, configurationUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglOpenCPACSConfiguration(tixiHandle, configurationUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'tixiHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'configurationUID' must be a string.\n");
    }

    tixiHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &configurationUID);

    ret = tiglOpenCPACSConfiguration(tixiHandle, configurationUID, &cpacsHandlePtr);
    handleError("tiglOpenCPACSConfiguration", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)cpacsHandlePtr;
}

void mex_tiglSaveCPACSConfiguration(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    char* configurationUID = NULL;
    TiglCPACSConfigurationHandle cpacsHandle;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglSaveCPACSConfiguration(configurationUID, cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglSaveCPACSConfiguration(configurationUID, cpacsHandle): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!mxIsChar(prhs[1])) {
        mexErrMsgTxt("Argument 'configurationUID' must be a string.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    mxToString(prhs[1], &configurationUID);
    cpacsHandle = mxToInt(prhs[2]);

    ret = tiglSaveCPACSConfiguration(configurationUID, cpacsHandle);
    handleError("tiglSaveCPACSConfiguration", ret);

}

void mex_tiglCloseCPACSConfiguration(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglCloseCPACSConfiguration(cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglCloseCPACSConfiguration(cpacsHandle): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);

    ret = tiglCloseCPACSConfiguration(cpacsHandle);
    handleError("tiglCloseCPACSConfiguration", ret);

}

void mex_tiglGetCPACSTixiHandle(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;

    /* output variables */
    TixiDocumentHandle tixiHandlePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglGetCPACSTixiHandle(cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetCPACSTixiHandle(cpacsHandle): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);

    ret = tiglGetCPACSTixiHandle(cpacsHandle, &tixiHandlePtr);
    handleError("tiglGetCPACSTixiHandle", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)tixiHandlePtr;
}

void mex_tiglIsCPACSConfigurationHandleValid(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;

    /* output variables */
    TiglBoolean isValidPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglIsCPACSConfigurationHandleValid(cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglIsCPACSConfigurationHandleValid(cpacsHandle): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);

    ret = tiglIsCPACSConfigurationHandleValid(cpacsHandle, &isValidPtr);
    handleError("tiglIsCPACSConfigurationHandleValid", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)isValidPtr;
}

void mex_tiglGetVersion(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    char* ret = NULL;

    /* check for corect number of in and out args */
    if (nrhs != 1) {
        mexErrMsgTxt("tiglGetVersion(): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetVersion(): Wrong number of output values. This function returns 1 value(s)\n");
    }


    ret = tiglGetVersion();

    plhs[0] = mxCreateString(ret);
}

void mex_tiglGetWingCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;

    /* output variables */
    int wingCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglGetWingCount(cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetWingCount(cpacsHandle): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);

    ret = tiglGetWingCount(cpacsHandle, &wingCountPtr);
    handleError("tiglGetWingCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)wingCountPtr;
}

void mex_tiglWingGetSegmentCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;

    /* output variables */
    int segmentCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetSegmentCount(cpacsHandle, wingIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSegmentCount(cpacsHandle, wingIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);

    ret = tiglWingGetSegmentCount(cpacsHandle, wingIndex, &segmentCountPtr);
    handleError("tiglWingGetSegmentCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentCountPtr;
}

void mex_tiglWingGetComponentSegmentCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;

    /* output variables */
    int compSegmentCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetComponentSegmentCount(cpacsHandle, wingIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetComponentSegmentCount(cpacsHandle, wingIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);

    ret = tiglWingGetComponentSegmentCount(cpacsHandle, wingIndex, &compSegmentCountPtr);
    handleError("tiglWingGetComponentSegmentCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)compSegmentCountPtr;
}

void mex_tiglWingGetComponentSegmentUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int compSegmentIndex;

    /* output variables */
    char* uidNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetComponentSegmentUID(cpacsHandle, wingIndex, compSegmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetComponentSegmentUID(cpacsHandle, wingIndex, compSegmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'compSegmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    compSegmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetComponentSegmentUID(cpacsHandle, wingIndex, compSegmentIndex, &uidNamePtr);
    handleError("tiglWingGetComponentSegmentUID", ret);

    plhs[0] = mxCreateString(uidNamePtr);
}

void mex_tiglWingGetComponentSegmentIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    char* compSegmentUID = NULL;

    /* output variables */
    int segmentIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetComponentSegmentIndex(cpacsHandle, wingIndex, compSegmentUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetComponentSegmentIndex(cpacsHandle, wingIndex, compSegmentUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'compSegmentUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    mxToString(prhs[3], &compSegmentUID);

    ret = tiglWingGetComponentSegmentIndex(cpacsHandle, wingIndex, compSegmentUID, &segmentIndexPtr);
    handleError("tiglWingGetComponentSegmentIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentIndexPtr;
}

void mex_tiglWingGetUpperPoint(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    double eta;
    double xsi;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingGetUpperPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglWingGetUpperPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);

    ret = tiglWingGetUpperPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglWingGetUpperPoint", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglWingGetLowerPoint(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    double eta;
    double xsi;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingGetLowerPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglWingGetLowerPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);

    ret = tiglWingGetLowerPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglWingGetLowerPoint", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglWingSetGetPointBehavior(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    TiglGetPointBehavior behavior;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingSetGetPointBehavior(cpacsHandle, behavior): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglWingSetGetPointBehavior(cpacsHandle, behavior): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'behavior' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    behavior = mxToInt(prhs[2]);

    ret = tiglWingSetGetPointBehavior(cpacsHandle, behavior);
    handleError("tiglWingSetGetPointBehavior", ret);

}

void mex_tiglWingGetChordPoint(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    double eta;
    double xsi;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingGetChordPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglWingGetChordPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);

    ret = tiglWingGetChordPoint(cpacsHandle, wingIndex, segmentIndex, eta, xsi, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglWingGetChordPoint", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglWingGetChordNormal(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    double eta;
    double xsi;

    /* output variables */
    double normalXPtr;
    double normalYPtr;
    double normalZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingGetChordNormal(cpacsHandle, wingIndex, segmentIndex, eta, xsi): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglWingGetChordNormal(cpacsHandle, wingIndex, segmentIndex, eta, xsi): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);

    ret = tiglWingGetChordNormal(cpacsHandle, wingIndex, segmentIndex, eta, xsi, &normalXPtr, &normalYPtr, &normalZPtr);
    handleError("tiglWingGetChordNormal", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = normalXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = normalYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = normalZPtr;
}

void mex_tiglWingGetUpperPointAtDirection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    double eta;
    double xsi;
    double dirx;
    double diry;
    double dirz;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;
    double errorDistance;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 9) {
        mexErrMsgTxt("tiglWingGetUpperPointAtDirection(cpacsHandle, wingIndex, segmentIndex, eta, xsi, dirx, diry, dirz): Wrong number of arguments\n");
    }
    if (nlhs != 4) {
        mexErrMsgTxt("tiglWingGetUpperPointAtDirection(cpacsHandle, wingIndex, segmentIndex, eta, xsi, dirx, diry, dirz): Wrong number of output values. This function returns 4 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'dirx' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'diry' must not be an array.\n");
    }

    if (!isscalar(prhs[8])) {
        mexErrMsgTxt("Argument 'dirz' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);
    dirx = *mxGetPr(prhs[6]);
    diry = *mxGetPr(prhs[7]);
    dirz = *mxGetPr(prhs[8]);

    ret = tiglWingGetUpperPointAtDirection(cpacsHandle, wingIndex, segmentIndex, eta, xsi, dirx, diry, dirz, &pointXPtr, &pointYPtr, &pointZPtr, &errorDistance);
    handleError("tiglWingGetUpperPointAtDirection", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
    plhs[3] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[3]) = errorDistance;
}

void mex_tiglWingGetLowerPointAtDirection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    double eta;
    double xsi;
    double dirx;
    double diry;
    double dirz;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;
    double errorDistance;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 9) {
        mexErrMsgTxt("tiglWingGetLowerPointAtDirection(cpacsHandle, wingIndex, segmentIndex, eta, xsi, dirx, diry, dirz): Wrong number of arguments\n");
    }
    if (nlhs != 4) {
        mexErrMsgTxt("tiglWingGetLowerPointAtDirection(cpacsHandle, wingIndex, segmentIndex, eta, xsi, dirx, diry, dirz): Wrong number of output values. This function returns 4 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'dirx' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'diry' must not be an array.\n");
    }

    if (!isscalar(prhs[8])) {
        mexErrMsgTxt("Argument 'dirz' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);
    dirx = *mxGetPr(prhs[6]);
    diry = *mxGetPr(prhs[7]);
    dirz = *mxGetPr(prhs[8]);

    ret = tiglWingGetLowerPointAtDirection(cpacsHandle, wingIndex, segmentIndex, eta, xsi, dirx, diry, dirz, &pointXPtr, &pointYPtr, &pointZPtr, &errorDistance);
    handleError("tiglWingGetLowerPointAtDirection", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
    plhs[3] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[3]) = errorDistance;
}

void mex_tiglWingGetSegmentEtaXsi(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    double pointX;
    double pointY;
    double pointZ;

    /* output variables */
    int segmentIndex;
    double eta;
    double xsi;
    int isOnTop;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingGetSegmentEtaXsi(cpacsHandle, wingIndex, pointX, pointY, pointZ): Wrong number of arguments\n");
    }
    if (nlhs != 4) {
        mexErrMsgTxt("tiglWingGetSegmentEtaXsi(cpacsHandle, wingIndex, pointX, pointY, pointZ): Wrong number of output values. This function returns 4 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'pointX' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'pointY' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'pointZ' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    pointX = *mxGetPr(prhs[3]);
    pointY = *mxGetPr(prhs[4]);
    pointZ = *mxGetPr(prhs[5]);

    ret = tiglWingGetSegmentEtaXsi(cpacsHandle, wingIndex, pointX, pointY, pointZ, &segmentIndex, &eta, &xsi, &isOnTop);
    handleError("tiglWingGetSegmentEtaXsi", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentIndex;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = eta;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = xsi;
    plhs[3] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[3]) = (double)isOnTop;
}

void mex_tiglWingGetInnerConnectedSegmentCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    int segmentCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetInnerConnectedSegmentCount(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetInnerConnectedSegmentCount(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetInnerConnectedSegmentCount(cpacsHandle, wingIndex, segmentIndex, &segmentCountPtr);
    handleError("tiglWingGetInnerConnectedSegmentCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentCountPtr;
}

void mex_tiglWingGetOuterConnectedSegmentCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    int segmentCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetOuterConnectedSegmentCount(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetOuterConnectedSegmentCount(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetOuterConnectedSegmentCount(cpacsHandle, wingIndex, segmentIndex, &segmentCountPtr);
    handleError("tiglWingGetOuterConnectedSegmentCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentCountPtr;
}

void mex_tiglWingGetInnerConnectedSegmentIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    int n;

    /* output variables */
    int connectedIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglWingGetInnerConnectedSegmentIndex(cpacsHandle, wingIndex, segmentIndex, n): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetInnerConnectedSegmentIndex(cpacsHandle, wingIndex, segmentIndex, n): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'n' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    n = mxToInt(prhs[4]);

    ret = tiglWingGetInnerConnectedSegmentIndex(cpacsHandle, wingIndex, segmentIndex, n, &connectedIndexPtr);
    handleError("tiglWingGetInnerConnectedSegmentIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)connectedIndexPtr;
}

void mex_tiglWingGetOuterConnectedSegmentIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    int n;

    /* output variables */
    int connectedIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglWingGetOuterConnectedSegmentIndex(cpacsHandle, wingIndex, segmentIndex, n): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetOuterConnectedSegmentIndex(cpacsHandle, wingIndex, segmentIndex, n): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'n' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    n = mxToInt(prhs[4]);

    ret = tiglWingGetOuterConnectedSegmentIndex(cpacsHandle, wingIndex, segmentIndex, n, &connectedIndexPtr);
    handleError("tiglWingGetOuterConnectedSegmentIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)connectedIndexPtr;
}

void mex_tiglWingGetInnerSectionAndElementIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    int sectionIndexPtr;
    int elementIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetInnerSectionAndElementIndex(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingGetInnerSectionAndElementIndex(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetInnerSectionAndElementIndex(cpacsHandle, wingIndex, segmentIndex, &sectionIndexPtr, &elementIndexPtr);
    handleError("tiglWingGetInnerSectionAndElementIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)sectionIndexPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)elementIndexPtr;
}

void mex_tiglWingGetOuterSectionAndElementIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    int sectionIndexPtr;
    int elementIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetOuterSectionAndElementIndex(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingGetOuterSectionAndElementIndex(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetOuterSectionAndElementIndex(cpacsHandle, wingIndex, segmentIndex, &sectionIndexPtr, &elementIndexPtr);
    handleError("tiglWingGetOuterSectionAndElementIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)sectionIndexPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)elementIndexPtr;
}

void mex_tiglWingGetInnerSectionAndElementUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    char* sectionUIDPtr = NULL;
    char* elementUIDPtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetInnerSectionAndElementUID(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingGetInnerSectionAndElementUID(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetInnerSectionAndElementUID(cpacsHandle, wingIndex, segmentIndex, &sectionUIDPtr, &elementUIDPtr);
    handleError("tiglWingGetInnerSectionAndElementUID", ret);

    plhs[0] = mxCreateString(sectionUIDPtr);
    plhs[1] = mxCreateString(elementUIDPtr);
}

void mex_tiglWingGetOuterSectionAndElementUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    char* sectionUIDPtr = NULL;
    char* elementUIDPtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetOuterSectionAndElementUID(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingGetOuterSectionAndElementUID(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetOuterSectionAndElementUID(cpacsHandle, wingIndex, segmentIndex, &sectionUIDPtr, &elementUIDPtr);
    handleError("tiglWingGetOuterSectionAndElementUID", ret);

    plhs[0] = mxCreateString(sectionUIDPtr);
    plhs[1] = mxCreateString(elementUIDPtr);
}

void mex_tiglWingGetProfileName(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int sectionIndex;
    int elementIndex;

    /* output variables */
    char* profileNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglWingGetProfileName(cpacsHandle, wingIndex, sectionIndex, elementIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetProfileName(cpacsHandle, wingIndex, sectionIndex, elementIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'sectionIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'elementIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    sectionIndex = mxToInt(prhs[3]);
    elementIndex = mxToInt(prhs[4]);

    ret = tiglWingGetProfileName(cpacsHandle, wingIndex, sectionIndex, elementIndex, &profileNamePtr);
    handleError("tiglWingGetProfileName", ret);

    plhs[0] = mxCreateString(profileNamePtr);
}

void mex_tiglWingGetUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;

    /* output variables */
    char* uidNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetUID(cpacsHandle, wingIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetUID(cpacsHandle, wingIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);

    ret = tiglWingGetUID(cpacsHandle, wingIndex, &uidNamePtr);
    handleError("tiglWingGetUID", ret);

    plhs[0] = mxCreateString(uidNamePtr);
}

void mex_tiglWingGetIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;

    /* output variables */
    int wingIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetIndex(cpacsHandle, wingUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetIndex(cpacsHandle, wingUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);

    ret = tiglWingGetIndex(cpacsHandle, wingUID, &wingIndexPtr);
    handleError("tiglWingGetIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)wingIndexPtr;
}

void mex_tiglWingGetSegmentUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    char* uidNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetSegmentUID(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSegmentUID(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetSegmentUID(cpacsHandle, wingIndex, segmentIndex, &uidNamePtr);
    handleError("tiglWingGetSegmentUID", ret);

    plhs[0] = mxCreateString(uidNamePtr);
}

void mex_tiglWingGetSegmentIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* segmentUID = NULL;

    /* output variables */
    int segmentIndexPtr;
    int wingIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetSegmentIndex(cpacsHandle, segmentUID): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingGetSegmentIndex(cpacsHandle, segmentUID): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'segmentUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &segmentUID);

    ret = tiglWingGetSegmentIndex(cpacsHandle, segmentUID, &segmentIndexPtr, &wingIndexPtr);
    handleError("tiglWingGetSegmentIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentIndexPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)wingIndexPtr;
}

void mex_tiglWingGetSectionCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;

    /* output variables */
    int sectionCount;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetSectionCount(cpacsHandle, wingIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSectionCount(cpacsHandle, wingIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);

    ret = tiglWingGetSectionCount(cpacsHandle, wingIndex, &sectionCount);
    handleError("tiglWingGetSectionCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)sectionCount;
}

void mex_tiglWingGetSectionUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int sectionIndex;

    /* output variables */
    char* uidNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetSectionUID(cpacsHandle, wingIndex, sectionIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSectionUID(cpacsHandle, wingIndex, sectionIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'sectionIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    sectionIndex = mxToInt(prhs[3]);

    ret = tiglWingGetSectionUID(cpacsHandle, wingIndex, sectionIndex, &uidNamePtr);
    handleError("tiglWingGetSectionUID", ret);

    plhs[0] = mxCreateString(uidNamePtr);
}

void mex_tiglWingGetSymmetry(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;

    /* output variables */
    TiglSymmetryAxis symmetryAxisPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetSymmetry(cpacsHandle, wingIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSymmetry(cpacsHandle, wingIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);

    ret = tiglWingGetSymmetry(cpacsHandle, wingIndex, &symmetryAxisPtr);
    handleError("tiglWingGetSymmetry", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)symmetryAxisPtr;
}

void mex_tiglWingComponentSegmentFindSegment(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;
    double x;
    double y;
    double z;

    /* output variables */
    char* segmentUID = NULL;
    char* wingUID = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingComponentSegmentFindSegment(cpacsHandle, componentSegmentUID, x, y, z): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingComponentSegmentFindSegment(cpacsHandle, componentSegmentUID, x, y, z): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'x' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'y' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'z' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);
    x = *mxGetPr(prhs[3]);
    y = *mxGetPr(prhs[4]);
    z = *mxGetPr(prhs[5]);

    ret = tiglWingComponentSegmentFindSegment(cpacsHandle, componentSegmentUID, x, y, z, &segmentUID, &wingUID);
    handleError("tiglWingComponentSegmentFindSegment", ret);

    plhs[0] = mxCreateString(segmentUID);
    plhs[1] = mxCreateString(wingUID);
}

void mex_tiglWingComponentSegmentGetPoint(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;
    double eta;
    double xsi;

    /* output variables */
    double x;
    double y;
    double z;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglWingComponentSegmentGetPoint(cpacsHandle, componentSegmentUID, eta, xsi): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglWingComponentSegmentGetPoint(cpacsHandle, componentSegmentUID, eta, xsi): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);
    eta = *mxGetPr(prhs[3]);
    xsi = *mxGetPr(prhs[4]);

    ret = tiglWingComponentSegmentGetPoint(cpacsHandle, componentSegmentUID, eta, xsi, &x, &y, &z);
    handleError("tiglWingComponentSegmentGetPoint", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = x;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = y;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = z;
}

void mex_tiglWingComponentSegmentPointGetEtaXsi(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;
    double pX;
    double pY;
    double pZ;

    /* output variables */
    double eta;
    double xsi;
    double errorDistance;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingComponentSegmentPointGetEtaXsi(cpacsHandle, componentSegmentUID, pX, pY, pZ): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglWingComponentSegmentPointGetEtaXsi(cpacsHandle, componentSegmentUID, pX, pY, pZ): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'pX' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'pY' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'pZ' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);
    pX = *mxGetPr(prhs[3]);
    pY = *mxGetPr(prhs[4]);
    pZ = *mxGetPr(prhs[5]);

    ret = tiglWingComponentSegmentPointGetEtaXsi(cpacsHandle, componentSegmentUID, pX, pY, pZ, &eta, &xsi, &errorDistance);
    handleError("tiglWingComponentSegmentPointGetEtaXsi", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = eta;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = xsi;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = errorDistance;
}

void mex_tiglWingComponentSegmentPointGetSegmentEtaXsi(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;
    double eta;
    double xsi;

    /* output variables */
    char* wingUID = NULL;
    char* segmentUID = NULL;
    double segmentEta;
    double segmentXsi;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglWingComponentSegmentPointGetSegmentEtaXsi(cpacsHandle, componentSegmentUID, eta, xsi): Wrong number of arguments\n");
    }
    if (nlhs != 4) {
        mexErrMsgTxt("tiglWingComponentSegmentPointGetSegmentEtaXsi(cpacsHandle, componentSegmentUID, eta, xsi): Wrong number of output values. This function returns 4 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);
    eta = *mxGetPr(prhs[3]);
    xsi = *mxGetPr(prhs[4]);

    ret = tiglWingComponentSegmentPointGetSegmentEtaXsi(cpacsHandle, componentSegmentUID, eta, xsi, &wingUID, &segmentUID, &segmentEta, &segmentXsi);
    handleError("tiglWingComponentSegmentPointGetSegmentEtaXsi", ret);

    plhs[0] = mxCreateString(wingUID);
    plhs[1] = mxCreateString(segmentUID);
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = segmentEta;
    plhs[3] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[3]) = segmentXsi;
}

void mex_tiglWingSegmentPointGetComponentSegmentEtaXsi(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* segmentUID = NULL;
    char* componentSegmentUID = NULL;
    double segmentEta;
    double segmentXsi;

    /* output variables */
    double eta;
    double xsi;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingSegmentPointGetComponentSegmentEtaXsi(cpacsHandle, segmentUID, componentSegmentUID, segmentEta, segmentXsi): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingSegmentPointGetComponentSegmentEtaXsi(cpacsHandle, segmentUID, componentSegmentUID, segmentEta, segmentXsi): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'segmentUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'segmentEta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'segmentXsi' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &segmentUID);
    mxToString(prhs[3], &componentSegmentUID);
    segmentEta = *mxGetPr(prhs[4]);
    segmentXsi = *mxGetPr(prhs[5]);

    ret = tiglWingSegmentPointGetComponentSegmentEtaXsi(cpacsHandle, segmentUID, componentSegmentUID, segmentEta, segmentXsi, &eta, &xsi);
    handleError("tiglWingSegmentPointGetComponentSegmentEtaXsi", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = eta;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = xsi;
}

void mex_tiglWingComponentSegmentGetSegmentIntersection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;
    char* segmentUID = NULL;
    double csEta1;
    double csXsi1;
    double csEta2;
    double csXsi2;
    double segmentEta;

    /* output variables */
    double segmentXsi;
    TiglBoolean hasWarning;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 9) {
        mexErrMsgTxt("tiglWingComponentSegmentGetSegmentIntersection(cpacsHandle, componentSegmentUID, segmentUID, csEta1, csXsi1, csEta2, csXsi2, segmentEta): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingComponentSegmentGetSegmentIntersection(cpacsHandle, componentSegmentUID, segmentUID, csEta1, csXsi1, csEta2, csXsi2, segmentEta): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'csEta1' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'csXsi1' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'csEta2' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'csXsi2' must not be an array.\n");
    }

    if (!isscalar(prhs[8])) {
        mexErrMsgTxt("Argument 'segmentEta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);
    mxToString(prhs[3], &segmentUID);
    csEta1 = *mxGetPr(prhs[4]);
    csXsi1 = *mxGetPr(prhs[5]);
    csEta2 = *mxGetPr(prhs[6]);
    csXsi2 = *mxGetPr(prhs[7]);
    segmentEta = *mxGetPr(prhs[8]);

    ret = tiglWingComponentSegmentGetSegmentIntersection(cpacsHandle, componentSegmentUID, segmentUID, csEta1, csXsi1, csEta2, csXsi2, segmentEta, &segmentXsi, &hasWarning);
    handleError("tiglWingComponentSegmentGetSegmentIntersection", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = segmentXsi;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)hasWarning;
}

void mex_tiglWingComponentSegmentComputeEtaIntersection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;
    double csEta1;
    double csXsi1;
    double csEta2;
    double csXsi2;
    double eta;

    /* output variables */
    double xsi;
    TiglBoolean hasWarning;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 8) {
        mexErrMsgTxt("tiglWingComponentSegmentComputeEtaIntersection(cpacsHandle, componentSegmentUID, csEta1, csXsi1, csEta2, csXsi2, eta): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingComponentSegmentComputeEtaIntersection(cpacsHandle, componentSegmentUID, csEta1, csXsi1, csEta2, csXsi2, eta): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'csEta1' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'csXsi1' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'csEta2' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'csXsi2' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);
    csEta1 = *mxGetPr(prhs[3]);
    csXsi1 = *mxGetPr(prhs[4]);
    csEta2 = *mxGetPr(prhs[5]);
    csXsi2 = *mxGetPr(prhs[6]);
    eta = *mxGetPr(prhs[7]);

    ret = tiglWingComponentSegmentComputeEtaIntersection(cpacsHandle, componentSegmentUID, csEta1, csXsi1, csEta2, csXsi2, eta, &xsi, &hasWarning);
    handleError("tiglWingComponentSegmentComputeEtaIntersection", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = xsi;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)hasWarning;
}

void mex_tiglWingInterpolateXsi(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* firstUID = NULL;
    double firstEta;
    double firstXsi;
    char* secondUID = NULL;
    double secondEta;
    double secondXsi;
    char* intersectionUID = NULL;
    double intersectionEta;

    /* output variables */
    double intersectionXsi;
    TiglBoolean hasWarning;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 10) {
        mexErrMsgTxt("tiglWingInterpolateXsi(cpacsHandle, firstUID, firstEta, firstXsi, secondUID, secondEta, secondXsi, intersectionUID, intersectionEta): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglWingInterpolateXsi(cpacsHandle, firstUID, firstEta, firstXsi, secondUID, secondEta, secondXsi, intersectionUID, intersectionEta): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'firstUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'firstEta' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'firstXsi' must not be an array.\n");
    }

    if (!mxIsChar(prhs[5])) {
        mexErrMsgTxt("Argument 'secondUID' must be a string.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'secondEta' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'secondXsi' must not be an array.\n");
    }

    if (!mxIsChar(prhs[8])) {
        mexErrMsgTxt("Argument 'intersectionUID' must be a string.\n");
    }

    if (!isscalar(prhs[9])) {
        mexErrMsgTxt("Argument 'intersectionEta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &firstUID);
    firstEta = *mxGetPr(prhs[3]);
    firstXsi = *mxGetPr(prhs[4]);
    mxToString(prhs[5], &secondUID);
    secondEta = *mxGetPr(prhs[6]);
    secondXsi = *mxGetPr(prhs[7]);
    mxToString(prhs[8], &intersectionUID);
    intersectionEta = *mxGetPr(prhs[9]);

    ret = tiglWingInterpolateXsi(cpacsHandle, firstUID, firstEta, firstXsi, secondUID, secondEta, secondXsi, intersectionUID, intersectionEta, &intersectionXsi, &hasWarning);
    handleError("tiglWingInterpolateXsi", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = intersectionXsi;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)hasWarning;
}

void mex_tiglWingComponentSegmentGetNumberOfSegments(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;

    /* output variables */
    int nsegments;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingComponentSegmentGetNumberOfSegments(cpacsHandle, componentSegmentUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingComponentSegmentGetNumberOfSegments(cpacsHandle, componentSegmentUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);

    ret = tiglWingComponentSegmentGetNumberOfSegments(cpacsHandle, componentSegmentUID, &nsegments);
    handleError("tiglWingComponentSegmentGetNumberOfSegments", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)nsegments;
}

void mex_tiglWingComponentSegmentGetSegmentUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;
    int segmentIndex;

    /* output variables */
    char* segmentUID = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingComponentSegmentGetSegmentUID(cpacsHandle, componentSegmentUID, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingComponentSegmentGetSegmentUID(cpacsHandle, componentSegmentUID, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingComponentSegmentGetSegmentUID(cpacsHandle, componentSegmentUID, segmentIndex, &segmentUID);
    handleError("tiglWingComponentSegmentGetSegmentUID", ret);

    plhs[0] = mxCreateString(segmentUID);
}

void mex_tiglWingGetSpan(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;

    /* output variables */
    double pSpan;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetSpan(cpacsHandle, wingUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSpan(cpacsHandle, wingUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);

    ret = tiglWingGetSpan(cpacsHandle, wingUID, &pSpan);
    handleError("tiglWingGetSpan", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pSpan;
}

void mex_tiglWingGetMAC(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;

    /* output variables */
    double mac_chord;
    double mac_x;
    double mac_y;
    double mac_z;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetMAC(cpacsHandle, wingUID): Wrong number of arguments\n");
    }
    if (nlhs != 4) {
        mexErrMsgTxt("tiglWingGetMAC(cpacsHandle, wingUID): Wrong number of output values. This function returns 4 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);

    ret = tiglWingGetMAC(cpacsHandle, wingUID, &mac_chord, &mac_x, &mac_y, &mac_z);
    handleError("tiglWingGetMAC", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = mac_chord;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = mac_x;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = mac_y;
    plhs[3] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[3]) = mac_z;
}

void mex_tiglGetControlSurfaceCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;

    /* output variables */
    int numControlSurfaces;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglGetControlSurfaceCount(cpacsHandle, componentSegmentUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetControlSurfaceCount(cpacsHandle, componentSegmentUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);

    ret = tiglGetControlSurfaceCount(cpacsHandle, componentSegmentUID, &numControlSurfaces);
    handleError("tiglGetControlSurfaceCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)numControlSurfaces;
}

void mex_tiglGetControlSurfaceUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentSegmentUID = NULL;
    int controlSurfaceIndex;

    /* output variables */
    char* controlSurfaceUID = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglGetControlSurfaceUID(cpacsHandle, componentSegmentUID, controlSurfaceIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetControlSurfaceUID(cpacsHandle, componentSegmentUID, controlSurfaceIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'controlSurfaceIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentSegmentUID);
    controlSurfaceIndex = mxToInt(prhs[3]);

    ret = tiglGetControlSurfaceUID(cpacsHandle, componentSegmentUID, controlSurfaceIndex, &controlSurfaceUID);
    handleError("tiglGetControlSurfaceUID", ret);

    plhs[0] = mxCreateString(controlSurfaceUID);
}

void mex_tiglGetControlSurfaceType(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;

    /* output variables */
    TiglControlSurfaceType controlSurfaceType;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglGetControlSurfaceType(cpacsHandle, controlSurfaceUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetControlSurfaceType(cpacsHandle, controlSurfaceUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);

    ret = tiglGetControlSurfaceType(cpacsHandle, controlSurfaceUID, &controlSurfaceType);
    handleError("tiglGetControlSurfaceType", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)controlSurfaceType;
}

void mex_tiglControlSurfaceGetMinimumControlParameter(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;

    /* output variables */
    double minParameter;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglControlSurfaceGetMinimumControlParameter(cpacsHandle, controlSurfaceUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglControlSurfaceGetMinimumControlParameter(cpacsHandle, controlSurfaceUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);

    ret = tiglControlSurfaceGetMinimumControlParameter(cpacsHandle, controlSurfaceUID, &minParameter);
    handleError("tiglControlSurfaceGetMinimumControlParameter", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = minParameter;
}

void mex_tiglControlSurfaceGetMinimumDeflection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;

    /* output variables */
    double minDeflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglControlSurfaceGetMinimumDeflection(cpacsHandle, controlSurfaceUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglControlSurfaceGetMinimumDeflection(cpacsHandle, controlSurfaceUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);

    ret = tiglControlSurfaceGetMinimumDeflection(cpacsHandle, controlSurfaceUID, &minDeflection);
    handleError("tiglControlSurfaceGetMinimumDeflection", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = minDeflection;
}

void mex_tiglControlSurfaceGetMaximumControlParameter(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;

    /* output variables */
    double maxParameter;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglControlSurfaceGetMaximumControlParameter(cpacsHandle, controlSurfaceUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglControlSurfaceGetMaximumControlParameter(cpacsHandle, controlSurfaceUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);

    ret = tiglControlSurfaceGetMaximumControlParameter(cpacsHandle, controlSurfaceUID, &maxParameter);
    handleError("tiglControlSurfaceGetMaximumControlParameter", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = maxParameter;
}

void mex_tiglControlSurfaceGetMaximumDeflection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;

    /* output variables */
    double maxDeflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglControlSurfaceGetMaximumDeflection(cpacsHandle, controlSurfaceUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglControlSurfaceGetMaximumDeflection(cpacsHandle, controlSurfaceUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);

    ret = tiglControlSurfaceGetMaximumDeflection(cpacsHandle, controlSurfaceUID, &maxDeflection);
    handleError("tiglControlSurfaceGetMaximumDeflection", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = maxDeflection;
}

void mex_tiglControlSurfaceGetControlParameter(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;

    /* output variables */
    double controlParameter;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglControlSurfaceGetControlParameter(cpacsHandle, controlSurfaceUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglControlSurfaceGetControlParameter(cpacsHandle, controlSurfaceUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);

    ret = tiglControlSurfaceGetControlParameter(cpacsHandle, controlSurfaceUID, &controlParameter);
    handleError("tiglControlSurfaceGetControlParameter", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = controlParameter;
}

void mex_tiglControlSurfaceGetDeflection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;

    /* output variables */
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglControlSurfaceGetDeflection(cpacsHandle, controlSurfaceUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglControlSurfaceGetDeflection(cpacsHandle, controlSurfaceUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);

    ret = tiglControlSurfaceGetDeflection(cpacsHandle, controlSurfaceUID, &deflection);
    handleError("tiglControlSurfaceGetDeflection", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = deflection;
}

void mex_tiglControlSurfaceSetControlParameter(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;
    double controlParameter;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglControlSurfaceSetControlParameter(cpacsHandle, controlSurfaceUID, controlParameter): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglControlSurfaceSetControlParameter(cpacsHandle, controlSurfaceUID, controlParameter): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'controlParameter' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);
    controlParameter = *mxGetPr(prhs[3]);

    ret = tiglControlSurfaceSetControlParameter(cpacsHandle, controlSurfaceUID, controlParameter);
    handleError("tiglControlSurfaceSetControlParameter", ret);

}

void mex_tiglControlSurfaceSetDeflection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* controlSurfaceUID = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglControlSurfaceSetDeflection(cpacsHandle, controlSurfaceUID, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglControlSurfaceSetDeflection(cpacsHandle, controlSurfaceUID, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'controlSurfaceUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &controlSurfaceUID);
    deflection = *mxGetPr(prhs[3]);

    ret = tiglControlSurfaceSetDeflection(cpacsHandle, controlSurfaceUID, deflection);
    handleError("tiglControlSurfaceSetDeflection", ret);

}

void mex_tiglGetFuselageCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;

    /* output variables */
    int fuselageCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglGetFuselageCount(cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetFuselageCount(cpacsHandle): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);

    ret = tiglGetFuselageCount(cpacsHandle, &fuselageCountPtr);
    handleError("tiglGetFuselageCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)fuselageCountPtr;
}

void mex_tiglFuselageGetSegmentCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;

    /* output variables */
    int segmentCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetSegmentCount(cpacsHandle, fuselageIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetSegmentCount(cpacsHandle, fuselageIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);

    ret = tiglFuselageGetSegmentCount(cpacsHandle, fuselageIndex, &segmentCountPtr);
    handleError("tiglFuselageGetSegmentCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentCountPtr;
}

void mex_tiglFuselageGetSectionCenter(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageSegmentUID = NULL;
    double eta;

    /* output variables */
    double pointX;
    double pointY;
    double pointZ;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetSectionCenter(cpacsHandle, fuselageSegmentUID, eta): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglFuselageGetSectionCenter(cpacsHandle, fuselageSegmentUID, eta): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageSegmentUID);
    eta = *mxGetPr(prhs[3]);

    ret = tiglFuselageGetSectionCenter(cpacsHandle, fuselageSegmentUID, eta, &pointX, &pointY, &pointZ);
    handleError("tiglFuselageGetSectionCenter", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointX;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointY;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZ;
}

void mex_tiglFuselageGetCrossSectionArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageSegmentUID = NULL;
    double eta;

    /* output variables */
    double area;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetCrossSectionArea(cpacsHandle, fuselageSegmentUID, eta): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetCrossSectionArea(cpacsHandle, fuselageSegmentUID, eta): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageSegmentUID);
    eta = *mxGetPr(prhs[3]);

    ret = tiglFuselageGetCrossSectionArea(cpacsHandle, fuselageSegmentUID, eta, &area);
    handleError("tiglFuselageGetCrossSectionArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = area;
}

void mex_tiglFuselageGetCenterLineLength(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageUID = NULL;

    /* output variables */
    double length;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetCenterLineLength(cpacsHandle, fuselageUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetCenterLineLength(cpacsHandle, fuselageUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageUID);

    ret = tiglFuselageGetCenterLineLength(cpacsHandle, fuselageUID, &length);
    handleError("tiglFuselageGetCenterLineLength", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = length;
}

void mex_tiglFuselageGetPoint(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    double eta;
    double zeta;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglFuselageGetPoint(cpacsHandle, fuselageIndex, segmentIndex, eta, zeta): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglFuselageGetPoint(cpacsHandle, fuselageIndex, segmentIndex, eta, zeta): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'zeta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    zeta = *mxGetPr(prhs[5]);

    ret = tiglFuselageGetPoint(cpacsHandle, fuselageIndex, segmentIndex, eta, zeta, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglFuselageGetPoint", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglFuselageSetGetPointBehavior(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    TiglGetPointBehavior behavior;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageSetGetPointBehavior(cpacsHandle, behavior): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglFuselageSetGetPointBehavior(cpacsHandle, behavior): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'behavior' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    behavior = mxToInt(prhs[2]);

    ret = tiglFuselageSetGetPointBehavior(cpacsHandle, behavior);
    handleError("tiglFuselageSetGetPointBehavior", ret);

}

void mex_tiglFuselageGetPointAngle(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    double eta;
    double alpha;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglFuselageGetPointAngle(cpacsHandle, fuselageIndex, segmentIndex, eta, alpha): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglFuselageGetPointAngle(cpacsHandle, fuselageIndex, segmentIndex, eta, alpha): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'alpha' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    alpha = *mxGetPr(prhs[5]);

    ret = tiglFuselageGetPointAngle(cpacsHandle, fuselageIndex, segmentIndex, eta, alpha, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglFuselageGetPointAngle", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglFuselageGetPointAngleTranslated(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    double eta;
    double alpha;
    double y_cs;
    double z_cs;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 8) {
        mexErrMsgTxt("tiglFuselageGetPointAngleTranslated(cpacsHandle, fuselageIndex, segmentIndex, eta, alpha, y_cs, z_cs): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglFuselageGetPointAngleTranslated(cpacsHandle, fuselageIndex, segmentIndex, eta, alpha, y_cs, z_cs): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'alpha' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'y_cs' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'z_cs' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    alpha = *mxGetPr(prhs[5]);
    y_cs = *mxGetPr(prhs[6]);
    z_cs = *mxGetPr(prhs[7]);

    ret = tiglFuselageGetPointAngleTranslated(cpacsHandle, fuselageIndex, segmentIndex, eta, alpha, y_cs, z_cs, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglFuselageGetPointAngleTranslated", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglFuselageGetPointOnXPlane(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    double eta;
    double xpos;
    int pointIndex;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 7) {
        mexErrMsgTxt("tiglFuselageGetPointOnXPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, xpos, pointIndex): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglFuselageGetPointOnXPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, xpos, pointIndex): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xpos' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'pointIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xpos = *mxGetPr(prhs[5]);
    pointIndex = mxToInt(prhs[6]);

    ret = tiglFuselageGetPointOnXPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, xpos, pointIndex, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglFuselageGetPointOnXPlane", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglFuselageGetPointOnYPlane(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    double eta;
    double ypos;
    int pointIndex;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 7) {
        mexErrMsgTxt("tiglFuselageGetPointOnYPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, ypos, pointIndex): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglFuselageGetPointOnYPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, ypos, pointIndex): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'ypos' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'pointIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    ypos = *mxGetPr(prhs[5]);
    pointIndex = mxToInt(prhs[6]);

    ret = tiglFuselageGetPointOnYPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, ypos, pointIndex, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglFuselageGetPointOnYPlane", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglFuselageGetNumPointsOnXPlane(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    double eta;
    double xpos;

    /* output variables */
    int numPointsPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglFuselageGetNumPointsOnXPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, xpos): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetNumPointsOnXPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, xpos): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xpos' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xpos = *mxGetPr(prhs[5]);

    ret = tiglFuselageGetNumPointsOnXPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, xpos, &numPointsPtr);
    handleError("tiglFuselageGetNumPointsOnXPlane", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)numPointsPtr;
}

void mex_tiglFuselageGetNumPointsOnYPlane(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    double eta;
    double ypos;

    /* output variables */
    int numPointsPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglFuselageGetNumPointsOnYPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, ypos): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetNumPointsOnYPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, ypos): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'ypos' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    ypos = *mxGetPr(prhs[5]);

    ret = tiglFuselageGetNumPointsOnYPlane(cpacsHandle, fuselageIndex, segmentIndex, eta, ypos, &numPointsPtr);
    handleError("tiglFuselageGetNumPointsOnYPlane", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)numPointsPtr;
}

void mex_tiglFuselageGetCircumference(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    double eta;

    /* output variables */
    double circumferencePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglFuselageGetCircumference(cpacsHandle, fuselageIndex, segmentIndex, eta): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetCircumference(cpacsHandle, fuselageIndex, segmentIndex, eta): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);

    ret = tiglFuselageGetCircumference(cpacsHandle, fuselageIndex, segmentIndex, eta, &circumferencePtr);
    handleError("tiglFuselageGetCircumference", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = circumferencePtr;
}

void mex_tiglFuselageGetStartConnectedSegmentCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    int segmentCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetStartConnectedSegmentCount(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetStartConnectedSegmentCount(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetStartConnectedSegmentCount(cpacsHandle, fuselageIndex, segmentIndex, &segmentCountPtr);
    handleError("tiglFuselageGetStartConnectedSegmentCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentCountPtr;
}

void mex_tiglFuselageGetEndConnectedSegmentCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    int segmentCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetEndConnectedSegmentCount(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetEndConnectedSegmentCount(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetEndConnectedSegmentCount(cpacsHandle, fuselageIndex, segmentIndex, &segmentCountPtr);
    handleError("tiglFuselageGetEndConnectedSegmentCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentCountPtr;
}

void mex_tiglFuselageGetStartConnectedSegmentIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    int n;

    /* output variables */
    int connectedIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglFuselageGetStartConnectedSegmentIndex(cpacsHandle, fuselageIndex, segmentIndex, n): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetStartConnectedSegmentIndex(cpacsHandle, fuselageIndex, segmentIndex, n): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'n' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    n = mxToInt(prhs[4]);

    ret = tiglFuselageGetStartConnectedSegmentIndex(cpacsHandle, fuselageIndex, segmentIndex, n, &connectedIndexPtr);
    handleError("tiglFuselageGetStartConnectedSegmentIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)connectedIndexPtr;
}

void mex_tiglFuselageGetEndConnectedSegmentIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;
    int n;

    /* output variables */
    int connectedIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglFuselageGetEndConnectedSegmentIndex(cpacsHandle, fuselageIndex, segmentIndex, n): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetEndConnectedSegmentIndex(cpacsHandle, fuselageIndex, segmentIndex, n): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'n' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    n = mxToInt(prhs[4]);

    ret = tiglFuselageGetEndConnectedSegmentIndex(cpacsHandle, fuselageIndex, segmentIndex, n, &connectedIndexPtr);
    handleError("tiglFuselageGetEndConnectedSegmentIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)connectedIndexPtr;
}

void mex_tiglFuselageGetStartSectionAndElementUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    char* sectionUIDPtr = NULL;
    char* elementUIDPtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetStartSectionAndElementUID(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglFuselageGetStartSectionAndElementUID(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetStartSectionAndElementUID(cpacsHandle, fuselageIndex, segmentIndex, &sectionUIDPtr, &elementUIDPtr);
    handleError("tiglFuselageGetStartSectionAndElementUID", ret);

    plhs[0] = mxCreateString(sectionUIDPtr);
    plhs[1] = mxCreateString(elementUIDPtr);
}

void mex_tiglFuselageGetEndSectionAndElementUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    char* sectionUIDPtr = NULL;
    char* elementUIDPtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetEndSectionAndElementUID(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglFuselageGetEndSectionAndElementUID(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetEndSectionAndElementUID(cpacsHandle, fuselageIndex, segmentIndex, &sectionUIDPtr, &elementUIDPtr);
    handleError("tiglFuselageGetEndSectionAndElementUID", ret);

    plhs[0] = mxCreateString(sectionUIDPtr);
    plhs[1] = mxCreateString(elementUIDPtr);
}

void mex_tiglFuselageGetStartSectionAndElementIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    int sectionIndexPtr;
    int elementIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetStartSectionAndElementIndex(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglFuselageGetStartSectionAndElementIndex(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetStartSectionAndElementIndex(cpacsHandle, fuselageIndex, segmentIndex, &sectionIndexPtr, &elementIndexPtr);
    handleError("tiglFuselageGetStartSectionAndElementIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)sectionIndexPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)elementIndexPtr;
}

void mex_tiglFuselageGetEndSectionAndElementIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    int sectionIndexPtr;
    int elementIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetEndSectionAndElementIndex(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglFuselageGetEndSectionAndElementIndex(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetEndSectionAndElementIndex(cpacsHandle, fuselageIndex, segmentIndex, &sectionIndexPtr, &elementIndexPtr);
    handleError("tiglFuselageGetEndSectionAndElementIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)sectionIndexPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)elementIndexPtr;
}

void mex_tiglFuselageGetProfileName(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int sectionIndex;
    int elementIndex;

    /* output variables */
    char* profileNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglFuselageGetProfileName(cpacsHandle, fuselageIndex, sectionIndex, elementIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetProfileName(cpacsHandle, fuselageIndex, sectionIndex, elementIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'sectionIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'elementIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    sectionIndex = mxToInt(prhs[3]);
    elementIndex = mxToInt(prhs[4]);

    ret = tiglFuselageGetProfileName(cpacsHandle, fuselageIndex, sectionIndex, elementIndex, &profileNamePtr);
    handleError("tiglFuselageGetProfileName", ret);

    plhs[0] = mxCreateString(profileNamePtr);
}

void mex_tiglFuselageGetUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;

    /* output variables */
    char* uidNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetUID(cpacsHandle, fuselageIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetUID(cpacsHandle, fuselageIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);

    ret = tiglFuselageGetUID(cpacsHandle, fuselageIndex, &uidNamePtr);
    handleError("tiglFuselageGetUID", ret);

    plhs[0] = mxCreateString(uidNamePtr);
}

void mex_tiglFuselageGetIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageUID = NULL;

    /* output variables */
    int fuselageIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetIndex(cpacsHandle, fuselageUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetIndex(cpacsHandle, fuselageUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageUID);

    ret = tiglFuselageGetIndex(cpacsHandle, fuselageUID, &fuselageIndexPtr);
    handleError("tiglFuselageGetIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)fuselageIndexPtr;
}

void mex_tiglFuselageGetSegmentUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    char* uidNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetSegmentUID(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetSegmentUID(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetSegmentUID(cpacsHandle, fuselageIndex, segmentIndex, &uidNamePtr);
    handleError("tiglFuselageGetSegmentUID", ret);

    plhs[0] = mxCreateString(uidNamePtr);
}

void mex_tiglFuselageGetSegmentIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* segmentUID = NULL;

    /* output variables */
    int segmentIndexPtr;
    int fuselageIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetSegmentIndex(cpacsHandle, segmentUID): Wrong number of arguments\n");
    }
    if (nlhs != 2) {
        mexErrMsgTxt("tiglFuselageGetSegmentIndex(cpacsHandle, segmentUID): Wrong number of output values. This function returns 2 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'segmentUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &segmentUID);

    ret = tiglFuselageGetSegmentIndex(cpacsHandle, segmentUID, &segmentIndexPtr, &fuselageIndexPtr);
    handleError("tiglFuselageGetSegmentIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)segmentIndexPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)fuselageIndexPtr;
}

void mex_tiglFuselageGetSectionCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;

    /* output variables */
    int sectionCount;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetSectionCount(cpacsHandle, fuselageIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetSectionCount(cpacsHandle, fuselageIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);

    ret = tiglFuselageGetSectionCount(cpacsHandle, fuselageIndex, &sectionCount);
    handleError("tiglFuselageGetSectionCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)sectionCount;
}

void mex_tiglFuselageGetSectionUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int sectionIndex;

    /* output variables */
    char* uidNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetSectionUID(cpacsHandle, fuselageIndex, sectionIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetSectionUID(cpacsHandle, fuselageIndex, sectionIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'sectionIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    sectionIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetSectionUID(cpacsHandle, fuselageIndex, sectionIndex, &uidNamePtr);
    handleError("tiglFuselageGetSectionUID", ret);

    plhs[0] = mxCreateString(uidNamePtr);
}

void mex_tiglFuselageGetSymmetry(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;

    /* output variables */
    TiglSymmetryAxis symmetryAxisPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetSymmetry(cpacsHandle, fuselageIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetSymmetry(cpacsHandle, fuselageIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);

    ret = tiglFuselageGetSymmetry(cpacsHandle, fuselageIndex, &symmetryAxisPtr);
    handleError("tiglFuselageGetSymmetry", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)symmetryAxisPtr;
}

void mex_tiglFuselageGetMinumumDistanceToGround(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageUID = NULL;
    double axisPntX;
    double axisPntY;
    double axisPntZ;
    double axisDirX;
    double axisDirY;
    double axisDirZ;
    double angle;

    /* output variables */
    double pointXPtr;
    double pointYPtr;
    double pointZPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 10) {
        mexErrMsgTxt("tiglFuselageGetMinumumDistanceToGround(cpacsHandle, fuselageUID, axisPntX, axisPntY, axisPntZ, axisDirX, axisDirY, axisDirZ, angle): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglFuselageGetMinumumDistanceToGround(cpacsHandle, fuselageUID, axisPntX, axisPntY, axisPntZ, axisDirX, axisDirY, axisDirZ, angle): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'axisPntX' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'axisPntY' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'axisPntZ' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'axisDirX' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'axisDirY' must not be an array.\n");
    }

    if (!isscalar(prhs[8])) {
        mexErrMsgTxt("Argument 'axisDirZ' must not be an array.\n");
    }

    if (!isscalar(prhs[9])) {
        mexErrMsgTxt("Argument 'angle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageUID);
    axisPntX = *mxGetPr(prhs[3]);
    axisPntY = *mxGetPr(prhs[4]);
    axisPntZ = *mxGetPr(prhs[5]);
    axisDirX = *mxGetPr(prhs[6]);
    axisDirY = *mxGetPr(prhs[7]);
    axisDirZ = *mxGetPr(prhs[8]);
    angle = *mxGetPr(prhs[9]);

    ret = tiglFuselageGetMinumumDistanceToGround(cpacsHandle, fuselageUID, axisPntX, axisPntY, axisPntZ, axisDirX, axisDirY, axisDirZ, angle, &pointXPtr, &pointYPtr, &pointZPtr);
    handleError("tiglFuselageGetMinumumDistanceToGround", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointXPtr;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointYPtr;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZPtr;
}

void mex_tiglGetRotorCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;

    /* output variables */
    int rotorCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglGetRotorCount(cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetRotorCount(cpacsHandle): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);

    ret = tiglGetRotorCount(cpacsHandle, &rotorCountPtr);
    handleError("tiglGetRotorCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)rotorCountPtr;
}

void mex_tiglRotorGetUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    char* uidNamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetUID(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetUID(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetUID(cpacsHandle, rotorIndex, &uidNamePtr);
    handleError("tiglRotorGetUID", ret);

    plhs[0] = mxCreateString(uidNamePtr);
}

void mex_tiglRotorGetIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* rotorUID = NULL;

    /* output variables */
    int rotorIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetIndex(cpacsHandle, rotorUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetIndex(cpacsHandle, rotorUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &rotorUID);

    ret = tiglRotorGetIndex(cpacsHandle, rotorUID, &rotorIndexPtr);
    handleError("tiglRotorGetIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)rotorIndexPtr;
}

void mex_tiglRotorGetRadius(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    double radiusPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetRadius(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetRadius(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetRadius(cpacsHandle, rotorIndex, &radiusPtr);
    handleError("tiglRotorGetRadius", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = radiusPtr;
}

void mex_tiglRotorGetReferenceArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    double referenceAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetReferenceArea(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetReferenceArea(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetReferenceArea(cpacsHandle, rotorIndex, &referenceAreaPtr);
    handleError("tiglRotorGetReferenceArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = referenceAreaPtr;
}

void mex_tiglRotorGetTotalBladePlanformArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    double totalBladePlanformAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetTotalBladePlanformArea(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetTotalBladePlanformArea(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetTotalBladePlanformArea(cpacsHandle, rotorIndex, &totalBladePlanformAreaPtr);
    handleError("tiglRotorGetTotalBladePlanformArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = totalBladePlanformAreaPtr;
}

void mex_tiglRotorGetSolidity(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    double solidityPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetSolidity(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetSolidity(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetSolidity(cpacsHandle, rotorIndex, &solidityPtr);
    handleError("tiglRotorGetSolidity", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = solidityPtr;
}

void mex_tiglRotorGetSurfaceArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    double surfaceAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetSurfaceArea(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetSurfaceArea(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetSurfaceArea(cpacsHandle, rotorIndex, &surfaceAreaPtr);
    handleError("tiglRotorGetSurfaceArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = surfaceAreaPtr;
}

void mex_tiglRotorGetVolume(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    double volumePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetVolume(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetVolume(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetVolume(cpacsHandle, rotorIndex, &volumePtr);
    handleError("tiglRotorGetVolume", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = volumePtr;
}

void mex_tiglRotorGetTipSpeed(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    double tipSpeedPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetTipSpeed(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetTipSpeed(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetTipSpeed(cpacsHandle, rotorIndex, &tipSpeedPtr);
    handleError("tiglRotorGetTipSpeed", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = tipSpeedPtr;
}

void mex_tiglRotorGetRotorBladeCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;

    /* output variables */
    int rotorBladeCountPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglRotorGetRotorBladeCount(cpacsHandle, rotorIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorGetRotorBladeCount(cpacsHandle, rotorIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);

    ret = tiglRotorGetRotorBladeCount(cpacsHandle, rotorIndex, &rotorBladeCountPtr);
    handleError("tiglRotorGetRotorBladeCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)rotorBladeCountPtr;
}

void mex_tiglRotorBladeGetWingIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;

    /* output variables */
    int wingIndexPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglRotorBladeGetWingIndex(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetWingIndex(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);

    ret = tiglRotorBladeGetWingIndex(cpacsHandle, rotorIndex, rotorBladeIndex, &wingIndexPtr);
    handleError("tiglRotorBladeGetWingIndex", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)wingIndexPtr;
}

void mex_tiglRotorBladeGetWingUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;

    /* output variables */
    char* wingUIDPtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglRotorBladeGetWingUID(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetWingUID(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);

    ret = tiglRotorBladeGetWingUID(cpacsHandle, rotorIndex, rotorBladeIndex, &wingUIDPtr);
    handleError("tiglRotorBladeGetWingUID", ret);

    plhs[0] = mxCreateString(wingUIDPtr);
}

void mex_tiglRotorBladeGetAzimuthAngle(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;

    /* output variables */
    double azimuthAnglePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglRotorBladeGetAzimuthAngle(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetAzimuthAngle(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);

    ret = tiglRotorBladeGetAzimuthAngle(cpacsHandle, rotorIndex, rotorBladeIndex, &azimuthAnglePtr);
    handleError("tiglRotorBladeGetAzimuthAngle", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = azimuthAnglePtr;
}

void mex_tiglRotorBladeGetRadius(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;

    /* output variables */
    double radiusPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglRotorBladeGetRadius(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetRadius(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);

    ret = tiglRotorBladeGetRadius(cpacsHandle, rotorIndex, rotorBladeIndex, &radiusPtr);
    handleError("tiglRotorBladeGetRadius", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = radiusPtr;
}

void mex_tiglRotorBladeGetPlanformArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;

    /* output variables */
    double planformAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglRotorBladeGetPlanformArea(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetPlanformArea(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);

    ret = tiglRotorBladeGetPlanformArea(cpacsHandle, rotorIndex, rotorBladeIndex, &planformAreaPtr);
    handleError("tiglRotorBladeGetPlanformArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = planformAreaPtr;
}

void mex_tiglRotorBladeGetSurfaceArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;

    /* output variables */
    double surfaceAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglRotorBladeGetSurfaceArea(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetSurfaceArea(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);

    ret = tiglRotorBladeGetSurfaceArea(cpacsHandle, rotorIndex, rotorBladeIndex, &surfaceAreaPtr);
    handleError("tiglRotorBladeGetSurfaceArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = surfaceAreaPtr;
}

void mex_tiglRotorBladeGetVolume(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;

    /* output variables */
    double volumePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglRotorBladeGetVolume(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetVolume(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);

    ret = tiglRotorBladeGetVolume(cpacsHandle, rotorIndex, rotorBladeIndex, &volumePtr);
    handleError("tiglRotorBladeGetVolume", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = volumePtr;
}

void mex_tiglRotorBladeGetTipSpeed(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;

    /* output variables */
    double tipSpeedPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglRotorBladeGetTipSpeed(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetTipSpeed(cpacsHandle, rotorIndex, rotorBladeIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);

    ret = tiglRotorBladeGetTipSpeed(cpacsHandle, rotorIndex, rotorBladeIndex, &tipSpeedPtr);
    handleError("tiglRotorBladeGetTipSpeed", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = tipSpeedPtr;
}

void mex_tiglRotorBladeGetLocalRadius(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;
    int segmentIndex;
    double eta;

    /* output variables */
    double radiusPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglRotorBladeGetLocalRadius(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetLocalRadius(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);
    segmentIndex = mxToInt(prhs[4]);
    eta = *mxGetPr(prhs[5]);

    ret = tiglRotorBladeGetLocalRadius(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta, &radiusPtr);
    handleError("tiglRotorBladeGetLocalRadius", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = radiusPtr;
}

void mex_tiglRotorBladeGetLocalChord(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;
    int segmentIndex;
    double eta;

    /* output variables */
    double chordPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglRotorBladeGetLocalChord(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetLocalChord(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);
    segmentIndex = mxToInt(prhs[4]);
    eta = *mxGetPr(prhs[5]);

    ret = tiglRotorBladeGetLocalChord(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta, &chordPtr);
    handleError("tiglRotorBladeGetLocalChord", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = chordPtr;
}

void mex_tiglRotorBladeGetLocalTwistAngle(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int rotorIndex;
    int rotorBladeIndex;
    int segmentIndex;
    double eta;

    /* output variables */
    double twistAnglePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglRotorBladeGetLocalTwistAngle(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglRotorBladeGetLocalTwistAngle(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'rotorIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'rotorBladeIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    rotorIndex = mxToInt(prhs[2]);
    rotorBladeIndex = mxToInt(prhs[3]);
    segmentIndex = mxToInt(prhs[4]);
    eta = *mxGetPr(prhs[5]);

    ret = tiglRotorBladeGetLocalTwistAngle(cpacsHandle, rotorIndex, rotorBladeIndex, segmentIndex, eta, &twistAnglePtr);
    handleError("tiglRotorBladeGetLocalTwistAngle", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = twistAnglePtr;
}

void mex_tiglIntersectComponents(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentUidOne = NULL;
    char* componentUidTwo = NULL;

    /* output variables */
    char* intersectionID = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglIntersectComponents(cpacsHandle, componentUidOne, componentUidTwo): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglIntersectComponents(cpacsHandle, componentUidOne, componentUidTwo): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentUidOne' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'componentUidTwo' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentUidOne);
    mxToString(prhs[3], &componentUidTwo);

    ret = tiglIntersectComponents(cpacsHandle, componentUidOne, componentUidTwo, &intersectionID);
    handleError("tiglIntersectComponents", ret);

    plhs[0] = mxCreateString(intersectionID);
}

void mex_tiglIntersectWithPlane(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentUid = NULL;
    double px;
    double py;
    double pz;
    double nx;
    double ny;
    double nz;

    /* output variables */
    char* intersectionID = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 9) {
        mexErrMsgTxt("tiglIntersectWithPlane(cpacsHandle, componentUid, px, py, pz, nx, ny, nz): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglIntersectWithPlane(cpacsHandle, componentUid, px, py, pz, nx, ny, nz): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentUid' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'px' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'py' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'pz' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'nx' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'ny' must not be an array.\n");
    }

    if (!isscalar(prhs[8])) {
        mexErrMsgTxt("Argument 'nz' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentUid);
    px = *mxGetPr(prhs[3]);
    py = *mxGetPr(prhs[4]);
    pz = *mxGetPr(prhs[5]);
    nx = *mxGetPr(prhs[6]);
    ny = *mxGetPr(prhs[7]);
    nz = *mxGetPr(prhs[8]);

    ret = tiglIntersectWithPlane(cpacsHandle, componentUid, px, py, pz, nx, ny, nz, &intersectionID);
    handleError("tiglIntersectWithPlane", ret);

    plhs[0] = mxCreateString(intersectionID);
}

void mex_tiglIntersectWithPlaneSegment(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentUid = NULL;
    double p1x;
    double p1y;
    double p1z;
    double p2x;
    double p2y;
    double p2z;
    double wx;
    double wy;
    double wz;

    /* output variables */
    char* intersectionID = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 12) {
        mexErrMsgTxt("tiglIntersectWithPlaneSegment(cpacsHandle, componentUid, p1x, p1y, p1z, p2x, p2y, p2z, wx, wy, wz): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglIntersectWithPlaneSegment(cpacsHandle, componentUid, p1x, p1y, p1z, p2x, p2y, p2z, wx, wy, wz): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentUid' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'p1x' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'p1y' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'p1z' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'p2x' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'p2y' must not be an array.\n");
    }

    if (!isscalar(prhs[8])) {
        mexErrMsgTxt("Argument 'p2z' must not be an array.\n");
    }

    if (!isscalar(prhs[9])) {
        mexErrMsgTxt("Argument 'wx' must not be an array.\n");
    }

    if (!isscalar(prhs[10])) {
        mexErrMsgTxt("Argument 'wy' must not be an array.\n");
    }

    if (!isscalar(prhs[11])) {
        mexErrMsgTxt("Argument 'wz' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentUid);
    p1x = *mxGetPr(prhs[3]);
    p1y = *mxGetPr(prhs[4]);
    p1z = *mxGetPr(prhs[5]);
    p2x = *mxGetPr(prhs[6]);
    p2y = *mxGetPr(prhs[7]);
    p2z = *mxGetPr(prhs[8]);
    wx = *mxGetPr(prhs[9]);
    wy = *mxGetPr(prhs[10]);
    wz = *mxGetPr(prhs[11]);

    ret = tiglIntersectWithPlaneSegment(cpacsHandle, componentUid, p1x, p1y, p1z, p2x, p2y, p2z, wx, wy, wz, &intersectionID);
    handleError("tiglIntersectWithPlaneSegment", ret);

    plhs[0] = mxCreateString(intersectionID);
}

void mex_tiglGetCurveIntersection(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* curvesID1 = NULL;
    int curve1Idx;
    char* curvesID2 = NULL;
    int curve2Idx;
    double tolerance;

    /* output variables */
    char* intersectionID = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 7) {
        mexErrMsgTxt("tiglGetCurveIntersection(cpacsHandle, curvesID1, curve1Idx, curvesID2, curve2Idx, tolerance): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetCurveIntersection(cpacsHandle, curvesID1, curve1Idx, curvesID2, curve2Idx, tolerance): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'curvesID1' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'curve1Idx' must not be an array.\n");
    }

    if (!mxIsChar(prhs[4])) {
        mexErrMsgTxt("Argument 'curvesID2' must be a string.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'curve2Idx' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'tolerance' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &curvesID1);
    curve1Idx = mxToInt(prhs[3]);
    mxToString(prhs[4], &curvesID2);
    curve2Idx = mxToInt(prhs[5]);
    tolerance = *mxGetPr(prhs[6]);

    ret = tiglGetCurveIntersection(cpacsHandle, curvesID1, curve1Idx, curvesID2, curve2Idx, tolerance, &intersectionID);
    handleError("tiglGetCurveIntersection", ret);

    plhs[0] = mxCreateString(intersectionID);
}

void mex_tiglIntersectGetLineCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* intersectionID = NULL;

    /* output variables */
    int lineCount;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglIntersectGetLineCount(cpacsHandle, intersectionID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglIntersectGetLineCount(cpacsHandle, intersectionID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'intersectionID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &intersectionID);

    ret = tiglIntersectGetLineCount(cpacsHandle, intersectionID, &lineCount);
    handleError("tiglIntersectGetLineCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)lineCount;
}

void mex_tiglGetCurveIntersectionCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* intersectionID = NULL;

    /* output variables */
    int pointCount;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglGetCurveIntersectionCount(cpacsHandle, intersectionID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetCurveIntersectionCount(cpacsHandle, intersectionID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'intersectionID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &intersectionID);

    ret = tiglGetCurveIntersectionCount(cpacsHandle, intersectionID, &pointCount);
    handleError("tiglGetCurveIntersectionCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)pointCount;
}

void mex_tiglIntersectGetPoint(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* intersectionID = NULL;
    int lineIdx;
    double eta;

    /* output variables */
    double pointX;
    double pointY;
    double pointZ;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglIntersectGetPoint(cpacsHandle, intersectionID, lineIdx, eta): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglIntersectGetPoint(cpacsHandle, intersectionID, lineIdx, eta): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'intersectionID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'lineIdx' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &intersectionID);
    lineIdx = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);

    ret = tiglIntersectGetPoint(cpacsHandle, intersectionID, lineIdx, eta, &pointX, &pointY, &pointZ);
    handleError("tiglIntersectGetPoint", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointX;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointY;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZ;
}

void mex_tiglGetCurveIntersectionPoint(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* intersectionID = NULL;
    int pointIdx;

    /* output variables */
    double pointX;
    double pointY;
    double pointZ;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglGetCurveIntersectionPoint(cpacsHandle, intersectionID, pointIdx): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglGetCurveIntersectionPoint(cpacsHandle, intersectionID, pointIdx): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'intersectionID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'pointIdx' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &intersectionID);
    pointIdx = mxToInt(prhs[3]);

    ret = tiglGetCurveIntersectionPoint(cpacsHandle, intersectionID, pointIdx, &pointX, &pointY, &pointZ);
    handleError("tiglGetCurveIntersectionPoint", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pointX;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = pointY;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = pointZ;
}

void mex_tiglGetCurveParameter(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* curveID = NULL;
    int curveIdx;
    double pointX;
    double pointY;
    double pointZ;

    /* output variables */
    double eta;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 7) {
        mexErrMsgTxt("tiglGetCurveParameter(cpacsHandle, curveID, curveIdx, pointX, pointY, pointZ): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetCurveParameter(cpacsHandle, curveID, curveIdx, pointX, pointY, pointZ): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'curveID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'curveIdx' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'pointX' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'pointY' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'pointZ' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &curveID);
    curveIdx = mxToInt(prhs[3]);
    pointX = *mxGetPr(prhs[4]);
    pointY = *mxGetPr(prhs[5]);
    pointZ = *mxGetPr(prhs[6]);

    ret = tiglGetCurveParameter(cpacsHandle, curveID, curveIdx, pointX, pointY, pointZ, &eta);
    handleError("tiglGetCurveParameter", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = eta;
}

void mex_tiglSetExportOptions(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    char* exporter_name = NULL;
    char* option_name = NULL;
    char* option_value = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglSetExportOptions(exporter_name, option_name, option_value): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglSetExportOptions(exporter_name, option_name, option_value): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!mxIsChar(prhs[1])) {
        mexErrMsgTxt("Argument 'exporter_name' must be a string.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'option_name' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'option_value' must be a string.\n");
    }

    mxToString(prhs[1], &exporter_name);
    mxToString(prhs[2], &option_name);
    mxToString(prhs[3], &option_value);

    ret = tiglSetExportOptions(exporter_name, option_name, option_value);
    handleError("tiglSetExportOptions", ret);

}

void mex_tiglExportComponent(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* uid = NULL;
    char* fileName = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportComponent(cpacsHandle, uid, fileName, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportComponent(cpacsHandle, uid, fileName, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'uid' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'fileName' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &uid);
    mxToString(prhs[3], &fileName);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportComponent(cpacsHandle, uid, fileName, deflection);
    handleError("tiglExportComponent", ret);

}

void mex_tiglExportConfiguration(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fileName = NULL;
    TiglBoolean fuseAllShapes;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportConfiguration(cpacsHandle, fileName, fuseAllShapes, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportConfiguration(cpacsHandle, fileName, fuseAllShapes, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fileName' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'fuseAllShapes' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fileName);
    fuseAllShapes = mxToInt(prhs[3]);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportConfiguration(cpacsHandle, fileName, fuseAllShapes, deflection);
    handleError("tiglExportConfiguration", ret);

}

void mex_tiglExportIGES(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* filenamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglExportIGES(cpacsHandle, filenamePtr): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportIGES(cpacsHandle, filenamePtr): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &filenamePtr);

    ret = tiglExportIGES(cpacsHandle, filenamePtr);
    handleError("tiglExportIGES", ret);

}

void mex_tiglExportFusedWingFuselageIGES(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* filenamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglExportFusedWingFuselageIGES(cpacsHandle, filenamePtr): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportFusedWingFuselageIGES(cpacsHandle, filenamePtr): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &filenamePtr);

    ret = tiglExportFusedWingFuselageIGES(cpacsHandle, filenamePtr);
    handleError("tiglExportFusedWingFuselageIGES", ret);

}

void mex_tiglExportSTEP(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* filenamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglExportSTEP(cpacsHandle, filenamePtr): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportSTEP(cpacsHandle, filenamePtr): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &filenamePtr);

    ret = tiglExportSTEP(cpacsHandle, filenamePtr);
    handleError("tiglExportSTEP", ret);

}

void mex_tiglExportFusedSTEP(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* filenamePtr = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglExportFusedSTEP(cpacsHandle, filenamePtr): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportFusedSTEP(cpacsHandle, filenamePtr): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &filenamePtr);

    ret = tiglExportFusedSTEP(cpacsHandle, filenamePtr);
    handleError("tiglExportFusedSTEP", ret);

}

void mex_tiglExportMeshedWingSTL(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedWingSTL(cpacsHandle, wingIndex, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedWingSTL(cpacsHandle, wingIndex, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedWingSTL(cpacsHandle, wingIndex, filenamePtr, deflection);
    handleError("tiglExportMeshedWingSTL", ret);

}

void mex_tiglExportMeshedWingSTLByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedWingSTLByUID(cpacsHandle, wingUID, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedWingSTLByUID(cpacsHandle, wingUID, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedWingSTLByUID(cpacsHandle, wingUID, filenamePtr, deflection);
    handleError("tiglExportMeshedWingSTLByUID", ret);

}

void mex_tiglExportMeshedFuselageSTL(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedFuselageSTL(cpacsHandle, fuselageIndex, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedFuselageSTL(cpacsHandle, fuselageIndex, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedFuselageSTL(cpacsHandle, fuselageIndex, filenamePtr, deflection);
    handleError("tiglExportMeshedFuselageSTL", ret);

}

void mex_tiglExportMeshedFuselageSTLByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageUID = NULL;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedFuselageSTLByUID(cpacsHandle, fuselageUID, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedFuselageSTLByUID(cpacsHandle, fuselageUID, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageUID);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedFuselageSTLByUID(cpacsHandle, fuselageUID, filenamePtr, deflection);
    handleError("tiglExportMeshedFuselageSTLByUID", ret);

}

void mex_tiglExportMeshedGeometrySTL(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglExportMeshedGeometrySTL(cpacsHandle, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedGeometrySTL(cpacsHandle, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &filenamePtr);
    deflection = *mxGetPr(prhs[3]);

    ret = tiglExportMeshedGeometrySTL(cpacsHandle, filenamePtr, deflection);
    handleError("tiglExportMeshedGeometrySTL", ret);

}

void mex_tiglExportMeshedWingVTKByIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedWingVTKByIndex(cpacsHandle, wingIndex, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedWingVTKByIndex(cpacsHandle, wingIndex, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedWingVTKByIndex(cpacsHandle, wingIndex, filenamePtr, deflection);
    handleError("tiglExportMeshedWingVTKByIndex", ret);

}

void mex_tiglExportMeshedWingVTKByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedWingVTKByUID(cpacsHandle, wingUID, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedWingVTKByUID(cpacsHandle, wingUID, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedWingVTKByUID(cpacsHandle, wingUID, filenamePtr, deflection);
    handleError("tiglExportMeshedWingVTKByUID", ret);

}

void mex_tiglExportMeshedFuselageVTKByIndex(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedFuselageVTKByIndex(cpacsHandle, fuselageIndex, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedFuselageVTKByIndex(cpacsHandle, fuselageIndex, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedFuselageVTKByIndex(cpacsHandle, fuselageIndex, filenamePtr, deflection);
    handleError("tiglExportMeshedFuselageVTKByIndex", ret);

}

void mex_tiglExportMeshedFuselageVTKByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageUID = NULL;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedFuselageVTKByUID(cpacsHandle, fuselageUID, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedFuselageVTKByUID(cpacsHandle, fuselageUID, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageUID);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedFuselageVTKByUID(cpacsHandle, fuselageUID, filenamePtr, deflection);
    handleError("tiglExportMeshedFuselageVTKByUID", ret);

}

void mex_tiglExportMeshedGeometryVTK(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglExportMeshedGeometryVTK(cpacsHandle, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedGeometryVTK(cpacsHandle, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &filenamePtr);
    deflection = *mxGetPr(prhs[3]);

    ret = tiglExportMeshedGeometryVTK(cpacsHandle, filenamePtr, deflection);
    handleError("tiglExportMeshedGeometryVTK", ret);

}

void mex_tiglExportMeshedWingVTKSimpleByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportMeshedWingVTKSimpleByUID(cpacsHandle, wingUID, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedWingVTKSimpleByUID(cpacsHandle, wingUID, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);
    mxToString(prhs[3], &filenamePtr);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportMeshedWingVTKSimpleByUID(cpacsHandle, wingUID, filenamePtr, deflection);
    handleError("tiglExportMeshedWingVTKSimpleByUID", ret);

}

void mex_tiglExportMeshedGeometryVTKSimple(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* filenamePtr = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglExportMeshedGeometryVTKSimple(cpacsHandle, filenamePtr, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportMeshedGeometryVTKSimple(cpacsHandle, filenamePtr, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'filenamePtr' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &filenamePtr);
    deflection = *mxGetPr(prhs[3]);

    ret = tiglExportMeshedGeometryVTKSimple(cpacsHandle, filenamePtr, deflection);
    handleError("tiglExportMeshedGeometryVTKSimple", ret);

}

void mex_tiglExportFuselageColladaByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageUID = NULL;
    char* filename = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportFuselageColladaByUID(cpacsHandle, fuselageUID, filename, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportFuselageColladaByUID(cpacsHandle, fuselageUID, filename, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filename' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageUID);
    mxToString(prhs[3], &filename);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportFuselageColladaByUID(cpacsHandle, fuselageUID, filename, deflection);
    handleError("tiglExportFuselageColladaByUID", ret);

}

void mex_tiglExportWingColladaByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;
    char* filename = NULL;
    double deflection;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 5) {
        mexErrMsgTxt("tiglExportWingColladaByUID(cpacsHandle, wingUID, filename, deflection): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportWingColladaByUID(cpacsHandle, wingUID, filename, deflection): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filename' must be a string.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'deflection' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);
    mxToString(prhs[3], &filename);
    deflection = *mxGetPr(prhs[4]);

    ret = tiglExportWingColladaByUID(cpacsHandle, wingUID, filename, deflection);
    handleError("tiglExportWingColladaByUID", ret);

}

void mex_tiglExportFusedBREP(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* filename = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglExportFusedBREP(cpacsHandle, filename): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportFusedBREP(cpacsHandle, filename): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'filename' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &filename);

    ret = tiglExportFusedBREP(cpacsHandle, filename);
    handleError("tiglExportFusedBREP", ret);

}

void mex_tiglExportFuselageBREPByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* fuselageUID = NULL;
    char* filename = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglExportFuselageBREPByUID(cpacsHandle, fuselageUID, filename): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportFuselageBREPByUID(cpacsHandle, fuselageUID, filename): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filename' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &fuselageUID);
    mxToString(prhs[3], &filename);

    ret = tiglExportFuselageBREPByUID(cpacsHandle, fuselageUID, filename);
    handleError("tiglExportFuselageBREPByUID", ret);

}

void mex_tiglExportWingBREPByUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;
    char* filename = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglExportWingBREPByUID(cpacsHandle, wingUID, filename): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglExportWingBREPByUID(cpacsHandle, wingUID, filename): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    if (!mxIsChar(prhs[3])) {
        mexErrMsgTxt("Argument 'filename' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);
    mxToString(prhs[3], &filename);

    ret = tiglExportWingBREPByUID(cpacsHandle, wingUID, filename);
    handleError("tiglExportWingBREPByUID", ret);

}

void mex_tiglWingComponentSegmentGetMaterialCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* compSegmentUID = NULL;
    TiglStructureType structureType;
    double eta;
    double xsi;

    /* output variables */
    int materialCount;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglWingComponentSegmentGetMaterialCount(cpacsHandle, compSegmentUID, structureType, eta, xsi): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingComponentSegmentGetMaterialCount(cpacsHandle, compSegmentUID, structureType, eta, xsi): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'compSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'structureType' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &compSegmentUID);
    structureType = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);

    ret = tiglWingComponentSegmentGetMaterialCount(cpacsHandle, compSegmentUID, structureType, eta, xsi, &materialCount);
    handleError("tiglWingComponentSegmentGetMaterialCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)materialCount;
}

void mex_tiglWingComponentSegmentGetMaterialUID(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* compSegmentUID = NULL;
    TiglStructureType structureType;
    double eta;
    double xsi;
    int materialIndex;

    /* output variables */
    char* uid = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 7) {
        mexErrMsgTxt("tiglWingComponentSegmentGetMaterialUID(cpacsHandle, compSegmentUID, structureType, eta, xsi, materialIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingComponentSegmentGetMaterialUID(cpacsHandle, compSegmentUID, structureType, eta, xsi, materialIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'compSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'structureType' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'materialIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &compSegmentUID);
    structureType = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);
    materialIndex = mxToInt(prhs[6]);

    ret = tiglWingComponentSegmentGetMaterialUID(cpacsHandle, compSegmentUID, structureType, eta, xsi, materialIndex, &uid);
    handleError("tiglWingComponentSegmentGetMaterialUID", ret);

    plhs[0] = mxCreateString(uid);
}

void mex_tiglWingComponentSegmentGetMaterialThickness(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* compSegmentUID = NULL;
    TiglStructureType structureType;
    double eta;
    double xsi;
    int materialIndex;

    /* output variables */
    double thickness;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 7) {
        mexErrMsgTxt("tiglWingComponentSegmentGetMaterialThickness(cpacsHandle, compSegmentUID, structureType, eta, xsi, materialIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingComponentSegmentGetMaterialThickness(cpacsHandle, compSegmentUID, structureType, eta, xsi, materialIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'compSegmentUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'structureType' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'materialIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &compSegmentUID);
    structureType = mxToInt(prhs[3]);
    eta = *mxGetPr(prhs[4]);
    xsi = *mxGetPr(prhs[5]);
    materialIndex = mxToInt(prhs[6]);

    ret = tiglWingComponentSegmentGetMaterialThickness(cpacsHandle, compSegmentUID, structureType, eta, xsi, materialIndex, &thickness);
    handleError("tiglWingComponentSegmentGetMaterialThickness", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = thickness;
}

void mex_tiglFuselageGetVolume(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;

    /* output variables */
    double volumePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetVolume(cpacsHandle, fuselageIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetVolume(cpacsHandle, fuselageIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);

    ret = tiglFuselageGetVolume(cpacsHandle, fuselageIndex, &volumePtr);
    handleError("tiglFuselageGetVolume", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = volumePtr;
}

void mex_tiglWingGetVolume(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;

    /* output variables */
    double volumePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetVolume(cpacsHandle, wingIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetVolume(cpacsHandle, wingIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);

    ret = tiglWingGetVolume(cpacsHandle, wingIndex, &volumePtr);
    handleError("tiglWingGetVolume", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = volumePtr;
}

void mex_tiglWingGetSegmentVolume(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    double volumePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetSegmentVolume(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSegmentVolume(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetSegmentVolume(cpacsHandle, wingIndex, segmentIndex, &volumePtr);
    handleError("tiglWingGetSegmentVolume", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = volumePtr;
}

void mex_tiglFuselageGetSegmentVolume(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    double volumePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetSegmentVolume(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetSegmentVolume(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetSegmentVolume(cpacsHandle, fuselageIndex, segmentIndex, &volumePtr);
    handleError("tiglFuselageGetSegmentVolume", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = volumePtr;
}

void mex_tiglWingGetSurfaceArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;

    /* output variables */
    double surfaceAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetSurfaceArea(cpacsHandle, wingIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSurfaceArea(cpacsHandle, wingIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);

    ret = tiglWingGetSurfaceArea(cpacsHandle, wingIndex, &surfaceAreaPtr);
    handleError("tiglWingGetSurfaceArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = surfaceAreaPtr;
}

void mex_tiglFuselageGetSurfaceArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;

    /* output variables */
    double surfaceAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglFuselageGetSurfaceArea(cpacsHandle, fuselageIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetSurfaceArea(cpacsHandle, fuselageIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);

    ret = tiglFuselageGetSurfaceArea(cpacsHandle, fuselageIndex, &surfaceAreaPtr);
    handleError("tiglFuselageGetSurfaceArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = surfaceAreaPtr;
}

void mex_tiglWingGetSegmentSurfaceArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;

    /* output variables */
    double surfaceAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetSegmentSurfaceArea(cpacsHandle, wingIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSegmentSurfaceArea(cpacsHandle, wingIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglWingGetSegmentSurfaceArea(cpacsHandle, wingIndex, segmentIndex, &surfaceAreaPtr);
    handleError("tiglWingGetSegmentSurfaceArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = surfaceAreaPtr;
}

void mex_tiglWingGetSegmentUpperSurfaceAreaTrimmed(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    double eta1;
    double xsi1;
    double eta2;
    double xsi2;
    double eta3;
    double xsi3;
    double eta4;
    double xsi4;

    /* output variables */
    double surfaceArea;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 12) {
        mexErrMsgTxt("tiglWingGetSegmentUpperSurfaceAreaTrimmed(cpacsHandle, wingIndex, segmentIndex, eta1, xsi1, eta2, xsi2, eta3, xsi3, eta4, xsi4): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSegmentUpperSurfaceAreaTrimmed(cpacsHandle, wingIndex, segmentIndex, eta1, xsi1, eta2, xsi2, eta3, xsi3, eta4, xsi4): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta1' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi1' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'eta2' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'xsi2' must not be an array.\n");
    }

    if (!isscalar(prhs[8])) {
        mexErrMsgTxt("Argument 'eta3' must not be an array.\n");
    }

    if (!isscalar(prhs[9])) {
        mexErrMsgTxt("Argument 'xsi3' must not be an array.\n");
    }

    if (!isscalar(prhs[10])) {
        mexErrMsgTxt("Argument 'eta4' must not be an array.\n");
    }

    if (!isscalar(prhs[11])) {
        mexErrMsgTxt("Argument 'xsi4' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta1 = *mxGetPr(prhs[4]);
    xsi1 = *mxGetPr(prhs[5]);
    eta2 = *mxGetPr(prhs[6]);
    xsi2 = *mxGetPr(prhs[7]);
    eta3 = *mxGetPr(prhs[8]);
    xsi3 = *mxGetPr(prhs[9]);
    eta4 = *mxGetPr(prhs[10]);
    xsi4 = *mxGetPr(prhs[11]);

    ret = tiglWingGetSegmentUpperSurfaceAreaTrimmed(cpacsHandle, wingIndex, segmentIndex, eta1, xsi1, eta2, xsi2, eta3, xsi3, eta4, xsi4, &surfaceArea);
    handleError("tiglWingGetSegmentUpperSurfaceAreaTrimmed", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = surfaceArea;
}

void mex_tiglWingGetSegmentLowerSurfaceAreaTrimmed(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    int segmentIndex;
    double eta1;
    double xsi1;
    double eta2;
    double xsi2;
    double eta3;
    double xsi3;
    double eta4;
    double xsi4;

    /* output variables */
    double surfaceArea;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 12) {
        mexErrMsgTxt("tiglWingGetSegmentLowerSurfaceAreaTrimmed(cpacsHandle, wingIndex, segmentIndex, eta1, xsi1, eta2, xsi2, eta3, xsi3, eta4, xsi4): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetSegmentLowerSurfaceAreaTrimmed(cpacsHandle, wingIndex, segmentIndex, eta1, xsi1, eta2, xsi2, eta3, xsi3, eta4, xsi4): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'eta1' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'xsi1' must not be an array.\n");
    }

    if (!isscalar(prhs[6])) {
        mexErrMsgTxt("Argument 'eta2' must not be an array.\n");
    }

    if (!isscalar(prhs[7])) {
        mexErrMsgTxt("Argument 'xsi2' must not be an array.\n");
    }

    if (!isscalar(prhs[8])) {
        mexErrMsgTxt("Argument 'eta3' must not be an array.\n");
    }

    if (!isscalar(prhs[9])) {
        mexErrMsgTxt("Argument 'xsi3' must not be an array.\n");
    }

    if (!isscalar(prhs[10])) {
        mexErrMsgTxt("Argument 'eta4' must not be an array.\n");
    }

    if (!isscalar(prhs[11])) {
        mexErrMsgTxt("Argument 'xsi4' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);
    eta1 = *mxGetPr(prhs[4]);
    xsi1 = *mxGetPr(prhs[5]);
    eta2 = *mxGetPr(prhs[6]);
    xsi2 = *mxGetPr(prhs[7]);
    eta3 = *mxGetPr(prhs[8]);
    xsi3 = *mxGetPr(prhs[9]);
    eta4 = *mxGetPr(prhs[10]);
    xsi4 = *mxGetPr(prhs[11]);

    ret = tiglWingGetSegmentLowerSurfaceAreaTrimmed(cpacsHandle, wingIndex, segmentIndex, eta1, xsi1, eta2, xsi2, eta3, xsi3, eta4, xsi4, &surfaceArea);
    handleError("tiglWingGetSegmentLowerSurfaceAreaTrimmed", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = surfaceArea;
}

void mex_tiglFuselageGetSegmentSurfaceArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int fuselageIndex;
    int segmentIndex;

    /* output variables */
    double surfaceAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglFuselageGetSegmentSurfaceArea(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglFuselageGetSegmentSurfaceArea(cpacsHandle, fuselageIndex, segmentIndex): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'fuselageIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'segmentIndex' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    fuselageIndex = mxToInt(prhs[2]);
    segmentIndex = mxToInt(prhs[3]);

    ret = tiglFuselageGetSegmentSurfaceArea(cpacsHandle, fuselageIndex, segmentIndex, &surfaceAreaPtr);
    handleError("tiglFuselageGetSegmentSurfaceArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = surfaceAreaPtr;
}

void mex_tiglWingGetReferenceArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    int wingIndex;
    TiglSymmetryAxis symPlane;

    /* output variables */
    double referenceAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglWingGetReferenceArea(cpacsHandle, wingIndex, symPlane): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetReferenceArea(cpacsHandle, wingIndex, symPlane): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingIndex' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'symPlane' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    wingIndex = mxToInt(prhs[2]);
    symPlane = mxToInt(prhs[3]);

    ret = tiglWingGetReferenceArea(cpacsHandle, wingIndex, symPlane, &referenceAreaPtr);
    handleError("tiglWingGetReferenceArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = referenceAreaPtr;
}

void mex_tiglWingGetWettedArea(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* wingUID = NULL;

    /* output variables */
    double wettedAreaPtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglWingGetWettedArea(cpacsHandle, wingUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglWingGetWettedArea(cpacsHandle, wingUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'wingUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &wingUID);

    ret = tiglWingGetWettedArea(cpacsHandle, wingUID, &wettedAreaPtr);
    handleError("tiglWingGetWettedArea", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = wettedAreaPtr;
}

void mex_tiglProfileGetBSplineCount(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* profileUID = NULL;

    /* output variables */
    int curveCount;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglProfileGetBSplineCount(cpacsHandle, profileUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglProfileGetBSplineCount(cpacsHandle, profileUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'profileUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &profileUID);

    ret = tiglProfileGetBSplineCount(cpacsHandle, profileUID, &curveCount);
    handleError("tiglProfileGetBSplineCount", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)curveCount;
}

void mex_tiglProfileGetBSplineDataSizes(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* profileUID = NULL;
    int curveid;

    /* output variables */
    int degree;
    int nControlPoints;
    int nKnots;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 4) {
        mexErrMsgTxt("tiglProfileGetBSplineDataSizes(cpacsHandle, profileUID, curveid): Wrong number of arguments\n");
    }
    if (nlhs != 3) {
        mexErrMsgTxt("tiglProfileGetBSplineDataSizes(cpacsHandle, profileUID, curveid): Wrong number of output values. This function returns 3 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'profileUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'curveid' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &profileUID);
    curveid = mxToInt(prhs[3]);

    ret = tiglProfileGetBSplineDataSizes(cpacsHandle, profileUID, curveid, &degree, &nControlPoints, &nKnots);
    handleError("tiglProfileGetBSplineDataSizes", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)degree;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = (double)nControlPoints;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = (double)nKnots;
}

void mex_tiglProfileGetBSplineData(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* profileUID = NULL;
    int curveid;
    int nControlPoints;
    int nKnots;

    /* output variables */
    double* cpx = NULL;
    double* cpy = NULL;
    double* cpz = NULL;
    double* knots = NULL;

    int _cpx_size, _cpy_size, _cpz_size, _knots_size;
    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglProfileGetBSplineData(cpacsHandle, profileUID, curveid, nControlPoints, nKnots): Wrong number of arguments\n");
    }
    if (nlhs != 4) {
        mexErrMsgTxt("tiglProfileGetBSplineData(cpacsHandle, profileUID, curveid, nControlPoints, nKnots): Wrong number of output values. This function returns 4 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'profileUID' must be a string.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'curveid' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'nControlPoints' must not be an array.\n");
    }

    if (!isscalar(prhs[5])) {
        mexErrMsgTxt("Argument 'nKnots' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &profileUID);
    curveid = mxToInt(prhs[3]);
    nControlPoints = mxToInt(prhs[4]);
    nKnots = mxToInt(prhs[5]);

    _cpx_size = 1 * nControlPoints;
    _cpy_size = 1 * nControlPoints;
    _cpz_size = 1 * nControlPoints;
    _knots_size = 1 * nKnots;
    cpx = mxMalloc(sizeof(double) * _cpx_size);
    cpy = mxMalloc(sizeof(double) * _cpy_size);
    cpz = mxMalloc(sizeof(double) * _cpz_size);
    knots = mxMalloc(sizeof(double) * _knots_size);
    ret = tiglProfileGetBSplineData(cpacsHandle, profileUID, curveid, nControlPoints, cpx, cpy, cpz, nKnots, knots);
    handleError("tiglProfileGetBSplineData", ret);

    plhs[0] = dArrayToMx(cpx, _cpx_size);
    plhs[1] = dArrayToMx(cpy, _cpy_size);
    plhs[2] = dArrayToMx(cpz, _cpz_size);
    plhs[3] = dArrayToMx(knots, _knots_size);
}

void mex_tiglLogToFileEnabled(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    char* filePrefix = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglLogToFileEnabled(filePrefix): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglLogToFileEnabled(filePrefix): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!mxIsChar(prhs[1])) {
        mexErrMsgTxt("Argument 'filePrefix' must be a string.\n");
    }

    mxToString(prhs[1], &filePrefix);

    ret = tiglLogToFileEnabled(filePrefix);
    handleError("tiglLogToFileEnabled", ret);

}

void mex_tiglLogToFileDisabled(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 1) {
        mexErrMsgTxt("tiglLogToFileDisabled(): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglLogToFileDisabled(): Wrong number of output values. This function returns 0 value(s)\n");
    }


    ret = tiglLogToFileDisabled();
    handleError("tiglLogToFileDisabled", ret);

}

void mex_tiglLogSetFileEnding(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    char* ending = NULL;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglLogSetFileEnding(ending): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglLogSetFileEnding(ending): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!mxIsChar(prhs[1])) {
        mexErrMsgTxt("Argument 'ending' must be a string.\n");
    }

    mxToString(prhs[1], &ending);

    ret = tiglLogSetFileEnding(ending);
    handleError("tiglLogSetFileEnding", ret);

}

void mex_tiglLogSetTimeInFilenameEnabled(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglBoolean enabled;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglLogSetTimeInFilenameEnabled(enabled): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglLogSetTimeInFilenameEnabled(enabled): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'enabled' must not be an array.\n");
    }

    enabled = mxToInt(prhs[1]);

    ret = tiglLogSetTimeInFilenameEnabled(enabled);
    handleError("tiglLogSetTimeInFilenameEnabled", ret);

}

void mex_tiglLogSetVerbosity(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglLogLevel level;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglLogSetVerbosity(level): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglLogSetVerbosity(level): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'level' must not be an array.\n");
    }

    level = mxToInt(prhs[1]);

    ret = tiglLogSetVerbosity(level);
    handleError("tiglLogSetVerbosity", ret);

}

void mex_tiglCheckPointInside(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    double px;
    double py;
    double pz;
    char* componentUID = NULL;

    /* output variables */
    TiglBoolean isInside;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 6) {
        mexErrMsgTxt("tiglCheckPointInside(cpacsHandle, px, py, pz, componentUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglCheckPointInside(cpacsHandle, px, py, pz, componentUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!isscalar(prhs[2])) {
        mexErrMsgTxt("Argument 'px' must not be an array.\n");
    }

    if (!isscalar(prhs[3])) {
        mexErrMsgTxt("Argument 'py' must not be an array.\n");
    }

    if (!isscalar(prhs[4])) {
        mexErrMsgTxt("Argument 'pz' must not be an array.\n");
    }

    if (!mxIsChar(prhs[5])) {
        mexErrMsgTxt("Argument 'componentUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    px = *mxGetPr(prhs[2]);
    py = *mxGetPr(prhs[3]);
    pz = *mxGetPr(prhs[4]);
    mxToString(prhs[5], &componentUID);

    ret = tiglCheckPointInside(cpacsHandle, px, py, pz, componentUID, &isInside);
    handleError("tiglCheckPointInside", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)isInside;
}

void mex_tiglComponentGetHashCode(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentUID = NULL;

    /* output variables */
    int hashCodePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglComponentGetHashCode(cpacsHandle, componentUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglComponentGetHashCode(cpacsHandle, componentUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentUID);

    ret = tiglComponentGetHashCode(cpacsHandle, componentUID, &hashCodePtr);
    handleError("tiglComponentGetHashCode", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)hashCodePtr;
}

void mex_tiglComponentGetType(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;
    char* componentUID = NULL;

    /* output variables */
    TiglGeometricComponentType typePtr;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 3) {
        mexErrMsgTxt("tiglComponentGetType(cpacsHandle, componentUID): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglComponentGetType(cpacsHandle, componentUID): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    if (!mxIsChar(prhs[2])) {
        mexErrMsgTxt("Argument 'componentUID' must be a string.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);
    mxToString(prhs[2], &componentUID);

    ret = tiglComponentGetType(cpacsHandle, componentUID, &typePtr);
    handleError("tiglComponentGetType", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = (double)typePtr;
}

void mex_tiglGetErrorString(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglReturnCode errorCode;

    char* ret = NULL;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglGetErrorString(errorCode): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglGetErrorString(errorCode): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'errorCode' must not be an array.\n");
    }

    errorCode = mxToInt(prhs[1]);

    ret = tiglGetErrorString(errorCode);

    plhs[0] = mxCreateString(ret);
}

void mex_tiglSetDebugDataDirectory(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    char* directory = NULL;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglSetDebugDataDirectory(directory): Wrong number of arguments\n");
    }
    if (nlhs != 0) {
        mexErrMsgTxt("tiglSetDebugDataDirectory(directory): Wrong number of output values. This function returns 0 value(s)\n");
    }

    if (!mxIsChar(prhs[1])) {
        mexErrMsgTxt("Argument 'directory' must be a string.\n");
    }

    mxToString(prhs[1], &directory);

    tiglSetDebugDataDirectory(directory);

}

void mex_tiglConfigurationGetLength(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;

    /* output variables */
    double pLength;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglConfigurationGetLength(cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 1) {
        mexErrMsgTxt("tiglConfigurationGetLength(cpacsHandle): Wrong number of output values. This function returns 1 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);

    ret = tiglConfigurationGetLength(cpacsHandle, &pLength);
    handleError("tiglConfigurationGetLength", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = pLength;
}

void mex_tiglConfigurationGetBoundingBox(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* input arguments */
    TiglCPACSConfigurationHandle cpacsHandle;

    /* output variables */
    double minX;
    double minY;
    double minZ;
    double maxX;
    double maxY;
    double maxZ;

    TiglReturnCode ret;

    /* check for corect number of in and out args */
    if (nrhs != 2) {
        mexErrMsgTxt("tiglConfigurationGetBoundingBox(cpacsHandle): Wrong number of arguments\n");
    }
    if (nlhs != 6) {
        mexErrMsgTxt("tiglConfigurationGetBoundingBox(cpacsHandle): Wrong number of output values. This function returns 6 value(s)\n");
    }

    if (!isscalar(prhs[1])) {
        mexErrMsgTxt("Argument 'cpacsHandle' must not be an array.\n");
    }

    cpacsHandle = mxToInt(prhs[1]);

    ret = tiglConfigurationGetBoundingBox(cpacsHandle, &minX, &minY, &minZ, &maxX, &maxY, &maxZ);
    handleError("tiglConfigurationGetBoundingBox", ret);

    plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[0]) = minX;
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[1]) = minY;
    plhs[2] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[2]) = minZ;
    plhs[3] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[3]) = maxX;
    plhs[4] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[4]) = maxY;
    plhs[5] = mxCreateDoubleMatrix(1, 1, mxREAL);
    *mxGetPr(plhs[5]) = maxZ;
}


#ifdef __cplusplus
extern "C" {
#endif


/** main entry point of MATLAB, deals as a function dispatcher */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

    char * functionName = NULL;
    if (nrhs < 1) {
        mexErrMsgTxt("No function argument given!");
        return;
    }

    mxToString(prhs[0],&functionName);

    if (strcmp(functionName, "tiglOpenCPACSConfiguration") == 0) {
        mex_tiglOpenCPACSConfiguration(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglSaveCPACSConfiguration") == 0) {
        mex_tiglSaveCPACSConfiguration(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglCloseCPACSConfiguration") == 0) {
        mex_tiglCloseCPACSConfiguration(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetCPACSTixiHandle") == 0) {
        mex_tiglGetCPACSTixiHandle(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglIsCPACSConfigurationHandleValid") == 0) {
        mex_tiglIsCPACSConfigurationHandleValid(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetVersion") == 0) {
        mex_tiglGetVersion(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetWingCount") == 0) {
        mex_tiglGetWingCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSegmentCount") == 0) {
        mex_tiglWingGetSegmentCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetComponentSegmentCount") == 0) {
        mex_tiglWingGetComponentSegmentCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetComponentSegmentUID") == 0) {
        mex_tiglWingGetComponentSegmentUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetComponentSegmentIndex") == 0) {
        mex_tiglWingGetComponentSegmentIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetUpperPoint") == 0) {
        mex_tiglWingGetUpperPoint(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetLowerPoint") == 0) {
        mex_tiglWingGetLowerPoint(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingSetGetPointBehavior") == 0) {
        mex_tiglWingSetGetPointBehavior(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetChordPoint") == 0) {
        mex_tiglWingGetChordPoint(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetChordNormal") == 0) {
        mex_tiglWingGetChordNormal(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetUpperPointAtDirection") == 0) {
        mex_tiglWingGetUpperPointAtDirection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetLowerPointAtDirection") == 0) {
        mex_tiglWingGetLowerPointAtDirection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSegmentEtaXsi") == 0) {
        mex_tiglWingGetSegmentEtaXsi(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetInnerConnectedSegmentCount") == 0) {
        mex_tiglWingGetInnerConnectedSegmentCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetOuterConnectedSegmentCount") == 0) {
        mex_tiglWingGetOuterConnectedSegmentCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetInnerConnectedSegmentIndex") == 0) {
        mex_tiglWingGetInnerConnectedSegmentIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetOuterConnectedSegmentIndex") == 0) {
        mex_tiglWingGetOuterConnectedSegmentIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetInnerSectionAndElementIndex") == 0) {
        mex_tiglWingGetInnerSectionAndElementIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetOuterSectionAndElementIndex") == 0) {
        mex_tiglWingGetOuterSectionAndElementIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetInnerSectionAndElementUID") == 0) {
        mex_tiglWingGetInnerSectionAndElementUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetOuterSectionAndElementUID") == 0) {
        mex_tiglWingGetOuterSectionAndElementUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetProfileName") == 0) {
        mex_tiglWingGetProfileName(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetUID") == 0) {
        mex_tiglWingGetUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetIndex") == 0) {
        mex_tiglWingGetIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSegmentUID") == 0) {
        mex_tiglWingGetSegmentUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSegmentIndex") == 0) {
        mex_tiglWingGetSegmentIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSectionCount") == 0) {
        mex_tiglWingGetSectionCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSectionUID") == 0) {
        mex_tiglWingGetSectionUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSymmetry") == 0) {
        mex_tiglWingGetSymmetry(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentFindSegment") == 0) {
        mex_tiglWingComponentSegmentFindSegment(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentGetPoint") == 0) {
        mex_tiglWingComponentSegmentGetPoint(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentPointGetEtaXsi") == 0) {
        mex_tiglWingComponentSegmentPointGetEtaXsi(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentPointGetSegmentEtaXsi") == 0) {
        mex_tiglWingComponentSegmentPointGetSegmentEtaXsi(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingSegmentPointGetComponentSegmentEtaXsi") == 0) {
        mex_tiglWingSegmentPointGetComponentSegmentEtaXsi(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentGetSegmentIntersection") == 0) {
        mex_tiglWingComponentSegmentGetSegmentIntersection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentComputeEtaIntersection") == 0) {
        mex_tiglWingComponentSegmentComputeEtaIntersection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingInterpolateXsi") == 0) {
        mex_tiglWingInterpolateXsi(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentGetNumberOfSegments") == 0) {
        mex_tiglWingComponentSegmentGetNumberOfSegments(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentGetSegmentUID") == 0) {
        mex_tiglWingComponentSegmentGetSegmentUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSpan") == 0) {
        mex_tiglWingGetSpan(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetMAC") == 0) {
        mex_tiglWingGetMAC(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetControlSurfaceCount") == 0) {
        mex_tiglGetControlSurfaceCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetControlSurfaceUID") == 0) {
        mex_tiglGetControlSurfaceUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetControlSurfaceType") == 0) {
        mex_tiglGetControlSurfaceType(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglControlSurfaceGetMinimumControlParameter") == 0) {
        mex_tiglControlSurfaceGetMinimumControlParameter(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglControlSurfaceGetMinimumDeflection") == 0) {
        mex_tiglControlSurfaceGetMinimumDeflection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglControlSurfaceGetMaximumControlParameter") == 0) {
        mex_tiglControlSurfaceGetMaximumControlParameter(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglControlSurfaceGetMaximumDeflection") == 0) {
        mex_tiglControlSurfaceGetMaximumDeflection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglControlSurfaceGetControlParameter") == 0) {
        mex_tiglControlSurfaceGetControlParameter(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglControlSurfaceGetDeflection") == 0) {
        mex_tiglControlSurfaceGetDeflection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglControlSurfaceSetControlParameter") == 0) {
        mex_tiglControlSurfaceSetControlParameter(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglControlSurfaceSetDeflection") == 0) {
        mex_tiglControlSurfaceSetDeflection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetFuselageCount") == 0) {
        mex_tiglGetFuselageCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSegmentCount") == 0) {
        mex_tiglFuselageGetSegmentCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSectionCenter") == 0) {
        mex_tiglFuselageGetSectionCenter(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetCrossSectionArea") == 0) {
        mex_tiglFuselageGetCrossSectionArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetCenterLineLength") == 0) {
        mex_tiglFuselageGetCenterLineLength(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetPoint") == 0) {
        mex_tiglFuselageGetPoint(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageSetGetPointBehavior") == 0) {
        mex_tiglFuselageSetGetPointBehavior(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetPointAngle") == 0) {
        mex_tiglFuselageGetPointAngle(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetPointAngleTranslated") == 0) {
        mex_tiglFuselageGetPointAngleTranslated(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetPointOnXPlane") == 0) {
        mex_tiglFuselageGetPointOnXPlane(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetPointOnYPlane") == 0) {
        mex_tiglFuselageGetPointOnYPlane(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetNumPointsOnXPlane") == 0) {
        mex_tiglFuselageGetNumPointsOnXPlane(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetNumPointsOnYPlane") == 0) {
        mex_tiglFuselageGetNumPointsOnYPlane(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetCircumference") == 0) {
        mex_tiglFuselageGetCircumference(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetStartConnectedSegmentCount") == 0) {
        mex_tiglFuselageGetStartConnectedSegmentCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetEndConnectedSegmentCount") == 0) {
        mex_tiglFuselageGetEndConnectedSegmentCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetStartConnectedSegmentIndex") == 0) {
        mex_tiglFuselageGetStartConnectedSegmentIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetEndConnectedSegmentIndex") == 0) {
        mex_tiglFuselageGetEndConnectedSegmentIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetStartSectionAndElementUID") == 0) {
        mex_tiglFuselageGetStartSectionAndElementUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetEndSectionAndElementUID") == 0) {
        mex_tiglFuselageGetEndSectionAndElementUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetStartSectionAndElementIndex") == 0) {
        mex_tiglFuselageGetStartSectionAndElementIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetEndSectionAndElementIndex") == 0) {
        mex_tiglFuselageGetEndSectionAndElementIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetProfileName") == 0) {
        mex_tiglFuselageGetProfileName(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetUID") == 0) {
        mex_tiglFuselageGetUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetIndex") == 0) {
        mex_tiglFuselageGetIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSegmentUID") == 0) {
        mex_tiglFuselageGetSegmentUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSegmentIndex") == 0) {
        mex_tiglFuselageGetSegmentIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSectionCount") == 0) {
        mex_tiglFuselageGetSectionCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSectionUID") == 0) {
        mex_tiglFuselageGetSectionUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSymmetry") == 0) {
        mex_tiglFuselageGetSymmetry(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetMinumumDistanceToGround") == 0) {
        mex_tiglFuselageGetMinumumDistanceToGround(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetRotorCount") == 0) {
        mex_tiglGetRotorCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetUID") == 0) {
        mex_tiglRotorGetUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetIndex") == 0) {
        mex_tiglRotorGetIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetRadius") == 0) {
        mex_tiglRotorGetRadius(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetReferenceArea") == 0) {
        mex_tiglRotorGetReferenceArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetTotalBladePlanformArea") == 0) {
        mex_tiglRotorGetTotalBladePlanformArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetSolidity") == 0) {
        mex_tiglRotorGetSolidity(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetSurfaceArea") == 0) {
        mex_tiglRotorGetSurfaceArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetVolume") == 0) {
        mex_tiglRotorGetVolume(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetTipSpeed") == 0) {
        mex_tiglRotorGetTipSpeed(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorGetRotorBladeCount") == 0) {
        mex_tiglRotorGetRotorBladeCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetWingIndex") == 0) {
        mex_tiglRotorBladeGetWingIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetWingUID") == 0) {
        mex_tiglRotorBladeGetWingUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetAzimuthAngle") == 0) {
        mex_tiglRotorBladeGetAzimuthAngle(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetRadius") == 0) {
        mex_tiglRotorBladeGetRadius(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetPlanformArea") == 0) {
        mex_tiglRotorBladeGetPlanformArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetSurfaceArea") == 0) {
        mex_tiglRotorBladeGetSurfaceArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetVolume") == 0) {
        mex_tiglRotorBladeGetVolume(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetTipSpeed") == 0) {
        mex_tiglRotorBladeGetTipSpeed(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetLocalRadius") == 0) {
        mex_tiglRotorBladeGetLocalRadius(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetLocalChord") == 0) {
        mex_tiglRotorBladeGetLocalChord(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglRotorBladeGetLocalTwistAngle") == 0) {
        mex_tiglRotorBladeGetLocalTwistAngle(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglIntersectComponents") == 0) {
        mex_tiglIntersectComponents(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglIntersectWithPlane") == 0) {
        mex_tiglIntersectWithPlane(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglIntersectWithPlaneSegment") == 0) {
        mex_tiglIntersectWithPlaneSegment(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetCurveIntersection") == 0) {
        mex_tiglGetCurveIntersection(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglIntersectGetLineCount") == 0) {
        mex_tiglIntersectGetLineCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetCurveIntersectionCount") == 0) {
        mex_tiglGetCurveIntersectionCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglIntersectGetPoint") == 0) {
        mex_tiglIntersectGetPoint(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetCurveIntersectionPoint") == 0) {
        mex_tiglGetCurveIntersectionPoint(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetCurveParameter") == 0) {
        mex_tiglGetCurveParameter(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglSetExportOptions") == 0) {
        mex_tiglSetExportOptions(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportComponent") == 0) {
        mex_tiglExportComponent(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportConfiguration") == 0) {
        mex_tiglExportConfiguration(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportIGES") == 0) {
        mex_tiglExportIGES(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportFusedWingFuselageIGES") == 0) {
        mex_tiglExportFusedWingFuselageIGES(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportSTEP") == 0) {
        mex_tiglExportSTEP(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportFusedSTEP") == 0) {
        mex_tiglExportFusedSTEP(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedWingSTL") == 0) {
        mex_tiglExportMeshedWingSTL(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedWingSTLByUID") == 0) {
        mex_tiglExportMeshedWingSTLByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedFuselageSTL") == 0) {
        mex_tiglExportMeshedFuselageSTL(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedFuselageSTLByUID") == 0) {
        mex_tiglExportMeshedFuselageSTLByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedGeometrySTL") == 0) {
        mex_tiglExportMeshedGeometrySTL(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedWingVTKByIndex") == 0) {
        mex_tiglExportMeshedWingVTKByIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedWingVTKByUID") == 0) {
        mex_tiglExportMeshedWingVTKByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedFuselageVTKByIndex") == 0) {
        mex_tiglExportMeshedFuselageVTKByIndex(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedFuselageVTKByUID") == 0) {
        mex_tiglExportMeshedFuselageVTKByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedGeometryVTK") == 0) {
        mex_tiglExportMeshedGeometryVTK(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedWingVTKSimpleByUID") == 0) {
        mex_tiglExportMeshedWingVTKSimpleByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportMeshedGeometryVTKSimple") == 0) {
        mex_tiglExportMeshedGeometryVTKSimple(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportFuselageColladaByUID") == 0) {
        mex_tiglExportFuselageColladaByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportWingColladaByUID") == 0) {
        mex_tiglExportWingColladaByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportFusedBREP") == 0) {
        mex_tiglExportFusedBREP(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportFuselageBREPByUID") == 0) {
        mex_tiglExportFuselageBREPByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglExportWingBREPByUID") == 0) {
        mex_tiglExportWingBREPByUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentGetMaterialCount") == 0) {
        mex_tiglWingComponentSegmentGetMaterialCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentGetMaterialUID") == 0) {
        mex_tiglWingComponentSegmentGetMaterialUID(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingComponentSegmentGetMaterialThickness") == 0) {
        mex_tiglWingComponentSegmentGetMaterialThickness(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetVolume") == 0) {
        mex_tiglFuselageGetVolume(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetVolume") == 0) {
        mex_tiglWingGetVolume(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSegmentVolume") == 0) {
        mex_tiglWingGetSegmentVolume(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSegmentVolume") == 0) {
        mex_tiglFuselageGetSegmentVolume(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSurfaceArea") == 0) {
        mex_tiglWingGetSurfaceArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSurfaceArea") == 0) {
        mex_tiglFuselageGetSurfaceArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSegmentSurfaceArea") == 0) {
        mex_tiglWingGetSegmentSurfaceArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSegmentUpperSurfaceAreaTrimmed") == 0) {
        mex_tiglWingGetSegmentUpperSurfaceAreaTrimmed(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetSegmentLowerSurfaceAreaTrimmed") == 0) {
        mex_tiglWingGetSegmentLowerSurfaceAreaTrimmed(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglFuselageGetSegmentSurfaceArea") == 0) {
        mex_tiglFuselageGetSegmentSurfaceArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetReferenceArea") == 0) {
        mex_tiglWingGetReferenceArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglWingGetWettedArea") == 0) {
        mex_tiglWingGetWettedArea(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglProfileGetBSplineCount") == 0) {
        mex_tiglProfileGetBSplineCount(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglProfileGetBSplineDataSizes") == 0) {
        mex_tiglProfileGetBSplineDataSizes(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglProfileGetBSplineData") == 0) {
        mex_tiglProfileGetBSplineData(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglLogToFileEnabled") == 0) {
        mex_tiglLogToFileEnabled(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglLogToFileDisabled") == 0) {
        mex_tiglLogToFileDisabled(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglLogSetFileEnding") == 0) {
        mex_tiglLogSetFileEnding(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglLogSetTimeInFilenameEnabled") == 0) {
        mex_tiglLogSetTimeInFilenameEnabled(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglLogSetVerbosity") == 0) {
        mex_tiglLogSetVerbosity(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglCheckPointInside") == 0) {
        mex_tiglCheckPointInside(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglComponentGetHashCode") == 0) {
        mex_tiglComponentGetHashCode(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglComponentGetType") == 0) {
        mex_tiglComponentGetType(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglGetErrorString") == 0) {
        mex_tiglGetErrorString(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglSetDebugDataDirectory") == 0) {
        mex_tiglSetDebugDataDirectory(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglConfigurationGetLength") == 0) {
        mex_tiglConfigurationGetLength(nlhs, plhs, nrhs, prhs);
    }
    else if (strcmp(functionName, "tiglConfigurationGetBoundingBox") == 0) {
        mex_tiglConfigurationGetBoundingBox(nlhs, plhs, nrhs, prhs);
    }
    else {
        // this is only executed if the function could not be identified
        char text[255];
        snprintf(text, 250, "%s is not a valid tigl function!\n", functionName);
        mxFree(functionName);
        mexErrMsgTxt(text);
    }
    mxFree(functionName);
}

#ifdef __cplusplus
} /* extern C */
#endif

